local core = require ( "core" )

local GameView = require ( "View.GameView" )
local GameModel = require ( "Model.GameModel" )
local BoardModel = require ( "Model.BoardModel" )

local Events = require ( "Events" )
local GameEvents = Events.E_GAME

local PlayerCtrl = require ( "Controller.PlayerCtrl" )

local AIPlayerCtrl = core.class ( PlayerCtrl )

function AIPlayerCtrl.name ()
    return "CPU"
end

function AIPlayerCtrl.turnLabel ()
    return "CPU'S TURN"
end

function AIPlayerCtrl:ctor ( model, view, index )
    self.super.ctor ( self, model, view, index )
end

function AIPlayerCtrl:activate ()
    self.super.activate ( self )
    self._view:setFaderVisible ( true, 0.3 )
    self:makeTurnAfterDelay ( 1.5 )
end

function AIPlayerCtrl:onDartsHit ( x, y )
    local model = self._model
    local view = self._view
    
    local hitResult, value = self.super.onDartsHit ( self, x, y )
    if hitResult == GameModel.HIT_WIN then
        app.queue:subscribe ( self, self.hmEndRound, GameEvents.DEFAULT_TAP )
        view:setAllowedGestures ( GameView.GESTURES.TAP )
    elseif hitResult == GameModel.HIT_COMMON then
        self:makeTurnAfterDelay ( 1.0 )
    end
end

function AIPlayerCtrl:hmEndRound ()
    app.queue:unsubscribe ( self, self.hmEndRound, GameEvents.DEFAULT_TAP )
    self.super.hmEndRound ( self )
end

function AIPlayerCtrl:makeTurnAfterDelay ( delay )
    self:performAfterDelay ( delay, AIPlayerCtrl._makeTurn )
end

function AIPlayerCtrl:_makeTurn ()
    local model = self._model
    local view = self._view
    
    local hitRF = model:board ():getDoubleOrSingleRF ( model:activePlayer ():currentScore ())
    local r, f = 0
    if hitRF then
        r = math.random () * ( hitRF.rMax - hitRF.rMin ) + hitRF.rMin
        f = math.random () * ( hitRF.fMax - hitRF.fMin ) + hitRF.fMin
    else
        local board = model:board ()
        local alpha = 0.0
        local maxR = board:getCircle ( BoardModel.R_DOUBLE_RING ).r * ( 1 - alpha ) + board:getCircle ( BoardModel.R_FULL ).r * alpha
        r = math.random () * maxR
        f = math.random () * math.pi * 2
    end
    local vx, vy =  math.random () * 100 - 50, GameView.MIN_DARTS_VELOCITY + math.random () * 100 - 50 
    local x, y = BoardModel.modelCoordinates ( r, f )
    x, y = view:boardModelToWorld ( x, y )
    vx, vy = view:modelVelocityToWorld ( vx, vy )
    view:dartsAiming ( x, y )
    view:endDartsAiming ( vx, vy )
end

function AIPlayerCtrl:deactivate ()
    self._view:setFaderVisible ( false, 0.3 )
    self.super.deactivate ( self )
end

return AIPlayerCtrl
