local core = require ( "core" )

local GameView = require ( "View.GameView" )
local GameModel = require ( "Model.GameModel" )

local Network = require ( "Network" )

local Events = require ( "Events" )
local GameEvents = Events.E_GAME
local NetworkEvents = Events.E_NETWORK

local PlayerCtrl = require ( "Controller.PlayerCtrl" )

local NetPlayerCtrl = core.class ( PlayerCtrl )

function NetPlayerCtrl.name ()
    return "FRIEND"
end

function NetPlayerCtrl.turnLabel ()
    return "FRIEND'S TURN"
end

function NetPlayerCtrl:ctor ( model, view, index )
    self.super.ctor ( self, model, view, index )
    app.queue:subscribe ( self, self.onNetEvent, NetworkEvents )
    self._view.sigDartsThrow:connect ( self, self.onDartsThrowInactive )
    self._queue = {}
    self._requestCnt = 0
end

function NetPlayerCtrl:activate ()
    self.super.activate ( self )
    self._view.sigDartsThrow:disconnect ( self, self.onDartsThrowInactive )
    self._view:setFaderVisible ( true, 0.3 )
    self:_makeRequest ()
end

function NetPlayerCtrl:_makeRequest ()
    self._requestCnt = self._requestCnt + 1
    self:_process ()
end

function NetPlayerCtrl:_process ()
    if #self._queue == 0 or not self.isActive then
        return
    end

    while self._requestCnt > 0 do
        self._requestCnt = self._requestCnt - 1

        local msg = table.remove ( self._queue, 1 )
        local event, args = msg.event, msg.args
        if event == NetworkEvents.THROW then
            self:_makeThrow ( args )
        end
    end
end

function NetPlayerCtrl:onNetEvent ( sender, event, args )
    if event == NetworkEvents.THROW then
        local msg = { event = event, args = args }
        table.insert ( self._queue, msg )
        self:_process ()
    end
end

function NetPlayerCtrl:onDartsThrowInactive ( x, y, vx, vy )
    local cx, cy = self._view:boardCenter ()
    Network.sendMessage ( Network.MT_THROW, { x = x - cx, y = y - cy, vx = vx, vy = vy })
end

function NetPlayerCtrl:onDartsHit ( x, y )
    local model = self._model
    local view = self._view
    
    local hitResult, value = self.super.onDartsHit ( self, x, y )
    
    if hitResult == GameModel.HIT_COMMON then
        self:performAfterDelay ( 1.0, self._makeRequest )
    elseif hitResult == GameModel.HIT_WIN then
        app.queue:subscribe ( self, self.hmEndRound, GameEvents.DEFAULT_TAP )
        view:setAllowedGestures ( GameView.GESTURES.TAP )
    end
    return hitResult, value
end

function NetPlayerCtrl:hmEndRound ()
    app.queue:unsubscribe ( self, self.hmEndRound, GameEvents.DEFAULT_TAP )
    self.super.hmEndRound ( self )
end

function NetPlayerCtrl:_makeThrow ( args )
    local model = self._model
    local view = self._view
    
    local cx, cy = view:boardCenter ()
    view:dartsAiming ( args.x + cx,  args.y + cy)
    view:endDartsAiming ( args.vx, args.vy )
end

function NetPlayerCtrl:deactivate ()
    self._view:setFaderVisible ( false, 0.3 )
    self._view.sigDartsThrow:connect ( self, self.onDartsThrowInactive )
    self.super.deactivate ( self )
end

return NetPlayerCtrl
