_M = {}

local function createAction ( rc, target, options )
  if type ( rc ) == "function" then
    local a = rc ( target, options )
    a:stop ()
    return a
  end
  return rc:create ( target, options )
end

_M.Chain =
{
    newFromRsrc = function ( _, cfg, target, options )
        local action = MOAIAction.new ()
    
        local children = {}
        for _, rc in ipairs ( cfg ) do
            local a = createAction ( rc, target, options )
            table.insert ( children, a )
        end
    
        for i = 1, #children do
            local currAction = children [ i ]
            local nextAction = children [ i + 1 ]
            if i == 1 then
                action:addChild ( currAction )
            end

            if nextAction then
                currAction:setListener ( MOAIAction.EVENT_STOP,
                    function ()
                        action:addChild ( nextAction )
                    end
                )
            elseif cfg.onDone then
                currAction:setListener ( MOAIAction.EVENT_STOP,
                    function ()
                        cfg.onDone ( target, options )
                    end
                )
            end
        end
        
        return action
  end;
}

_M.Split =
{
    newFromRsrc = function ( _, cfg, target, options )
        local action = MOAIAction.new ()
        for _, rc in ipairs ( cfg ) do
            local a = createAction ( rc, target, options )
            action:addChild ( a )
        end
        return action
    end;
}

_M.Wait =
{
    newFromRsrc = function ( _, cfg, target, options )
        local timer = MOAITimer.new ()
        local t = type ( cfg.t )
        local dt
        if t == "table" then
            dt = cfg.t
        elseif t == "function" then
            dt = cfg.t ( target, options )
        elseif t == "number" then
            dt = cfg.t
        end
        timer:setSpan ( dt )
        return timer
    end
}

_M.Timeline =
{
    newFromRsrc = function ( _, cfg, target, options )
        local count = #cfg
        assert ( count > 0 )
        
        local duration = cfg.duration
        local loopsCnt = cfg.loopsCnt or 1

        local currIdx = #cfg + 1
        local currLoop = 0
        local timer = MOAITimer.new ()
      
        local function onTimeout ()
            local currNode = cfg [ currIdx ]

            if not currNode then
                -- start next loop
                if currLoop == loopsCnt and loopsCnt > 0 then
                -- done
                    timer:stop ()
                    return
                end

                local dt = math.max ( cfg[ 1 ].t, 0.01 )
                timer:setSpan ( dt )
                currLoop, currIdx = currLoop + 1, 1
                return
            end

            local t, func = currNode.t, currNode [ 1 ]
            func ( target, options )
            currIdx = currIdx + 1
            if currIdx <= count then
                local dt = cfg [ currIdx ].t - t
                timer:setSpan ( dt )
            else
                if not duration then
                    timer:stop ()
                else
                    -- wait for loop to end
                    timer:setSpan ( duration - currNode.t )
                end
            end
        end

        timer:setMode ( MOAITimer.LOOP )
        timer:setListener ( MOAITimer.EVENT_TIMER_END_SPAN, onTimeout)
        
        -- start first loop
        onTimeout ()
        
        return timer
    end
}

return _M