local core = require ( "core" )

local loader = core.class ( Object )

function loader:newFromRsrc ( cfg, context )
    return self.new ( cfg ):loadFromRsrc ( self.defaultInstance ( cfg, context ), cfg, context)
end

local function normalizedKey ( key )
    return key:sub ( 1, 1 ):upper () .. key:sub ( 2 )
end

local function loaderFunction ( key )
    return "load" .. normalizedKey ( key )
end

local function setterFunction ( key )
    return "set" .. normalizedKey ( key )
end

local addChildFunction = "addChild"

function loader.tablePathCheck ( value, context )
    return type ( value ) == "string" and context [ value ] or value
end

function loader:systemKey ( key )
    self [ loaderFunction ( key )] = function () end
end

loader:systemKey "create"

function loader:loadFromRsrc ( instance, cfg, context )
    for key, val in pairs ( cfg ) do
        local val = val
        local func = nil
        if type ( val ) == "table" and val.create then
            val = val:create ( context )
        end
        if type ( key ) == "string" then
            func = self [ loaderFunction ( key )]
            if func then
                func ( self, instance, val, context )
            else 
                func = instance [ setterFunction ( key )]
                if func then
                    func ( instance, val )
                else
                    instance [ key ] = val
                end
            end
        else
            func = self [ addChildFunction ]
            if func then
                func ( self, instance, key, val, context )
            else 
                func = instance [ addChildFunction ]
                if func then
                    func ( instance, val )
                else
                    instance [ key ] = val
                end
            end
        end
    end
    return instance
end

-- overload this method in inherited classes
function loader:defaultInstance ()
    return {}
end

-- declare load methods
loader [ "load@ID" ] = function ( self, instance, val, context )
    context [ val ] = instance
end

return loader
