local core = require ( "core" )

local PlayerModel = core.class ( Object )

local Events = require ( "Events" )
local PlayerEvents = Events.E_PLAYER

local DEFAULT_START_SCORE = 301
local DEFAULT_AVAILABLE_THROWS = 3

function PlayerModel:ctor ( index )
    self._index = index
end

function PlayerModel:name ()
    return self._playerName
end

function PlayerModel:setName ( name )
    self._playerName = name
end

function PlayerModel:startSetup ( startScore, throwsPerTurn )
    local startScore = startScore or DEFAULT_START_SCORE
    local throws = throwsPerTurn or DEFAULT_AVAILABLE_THROWS
    self:_setActiveScore ( startScore )
    self._throwsPerTurn = throws
    self:_setAvailableThrows ( throws )
end

function PlayerModel:cleanup ()
    self:_setAvailableThrows ( 0 )
    self:_setActiveScore ( nil )
end

function PlayerModel:_setActiveScore ( score )
    if self._activeScore then
        self._activeScore = score
    else
        self._score = score
    end
    app.queue:postEvent ( self, PlayerEvents.SCORE_CHANGED )
end

function PlayerModel:_setAvailableThrows ( throws )
    self._availableThrows = throws
    app.queue:postEvent ( self, PlayerEvents.THROWS_CHANGED )
end

function PlayerModel:activate ( throws )
    assert ( not self._activeScore, "Player " .. self._index .. " already activated!" )
    self._activeScore = self._score
    app.queue:postEvent ( self, PlayerEvents.ACTIVATED )
end

function PlayerModel:recalcScore ( submit )
    local score = submit and self._activeScore or self._score
    self:_setActiveScore ( score )
end

function PlayerModel:deactivate ()
    assert ( self._activeScore, "Player " .. self._index .. " not activated!" )
    self._score = self._activeScore
    self._activeScore = nil
    self:_setAvailableThrows ( self._throwsPerTurn )
    app.queue:postEvent ( self, PlayerEvents.DEACTIVATED )
end

function PlayerModel:index ()
    return self._index
end

function PlayerModel:currentScore ()
    return self._activeScore or self._score
end

function PlayerModel:availableThrows ()
    return self._availableThrows
end

function PlayerModel:processHit ( value )
    self:_setActiveScore ( self._activeScore - value )
    self:_setAvailableThrows ( self._availableThrows - 1 )
    return self._activeScore, self._availableThrows
end

function PlayerModel:isActive ()
    return self._activeScore
end

return PlayerModel
