local core = require ( "core" )

local Events = require ( "Events" )
local NetEvents = Events.E_NETWORK

local Network = core.class ( Object )
{
    MT_NONE = "NONE",
    MT_READY = "READY",
    MT_START = "START",
    MT_THROW = "THROW",
    
    _buffer = "",
}

function Network.open ()
    local channel = MOAIP2PChannel
    --[[
    local meta = getmetatable ( MOAIP2PChannel )
    for k,v in pairs ( meta ) do
        log ( "%s = %s", k, tostring ( v ))
    end
    --]]
    if channel then
        Network._channel = channel
        Network._connectionMonitorThread = MOAICoroutine.new ()
        Network._connectionMonitorThread:run ( Network._connectionMonitorFunc )
    end
end

function Network.close ()
    if Network._channel then
        Network._connectionMonitorThread:stop ()
        Network._connectionMonitorThread = nil
        Network._channel = nil
    end
end

function Network._connectionMonitorFunc ()
    local prevState = MOAIP2PChannel.INVALID
    local channel = Network._channel
    local emptyFunc = function () end
    local actionFunc = emptyFunc
    local run = true
    while run do
        coroutine:yield ()
        
        local currState = channel.getState ()
        if prevState ~= currState then
            print ( 'State changed. Old state=' .. prevState .. ' new state=' .. currState )
            Network._currentState = currState
            if prevState == MOAIP2PChannel.INVALID then
                if currState == MOAIP2PChannel.DISCONNECTED then
                    -- server started
                    actionFunc = emptyFunc
                elseif currState == MOAIP2PChannel.CONNECTED then
                    -- client side connection
                    actionFunc = Network.receiveMessage
                    local peerId = channel.getPeerId ()
                    app.queue:postEvent ( nil, E_SYSTEM.SESSION_STARTED, { isHost = peerId == 0, id = peerId })
                end
            elseif prevState == MOAIP2PChannel.DISCONNECTED then
                if currState == MOAIP2PChannel.INVALID then
                    -- single player
                    actionFunc = emptyFunc
                    run = false
                elseif currState == MOAIP2PChannel.CONNECTED then
                    -- server side connection
                    actionFunc = Network.receiveMessage
                    local peerId = channel.getPeerId()
                    app.queue:postEvent ( nil, E_SYSTEM.SESSION_STARTED, { isHost = peerId == 0, id = peerId })
                end
            elseif prevState == MOAIP2PChannel.CONNECTED then
                if currState == MOAIP2PChannel.INVALID then
                    -- close game
                    actionFunc = emptyFunc
                    run = false
                elseif currState == MOAIP2PChannel.DISCONNECTED then
                    -- close game
                    actionFunc = emptyFunc
                    run = false
                end
            end
            prevState = currState
        end

        actionFunc ()
    end
end

function Network.isAvailable ()
    return Network._channel ~= nil
end

function Network.isConnected ()
    return Network._channel and Network._currentState == MOAIP2PChannel.CONNECTED
end

function Network.sendMessage ( mType, data )
    local data = data or {}
    local message = string.format ( "%s\n%s", mType, MOAIJsonParser.encode ( data ))
    Network._channel.send ( message )
    return message
end

function Network.receiveMessage ()
    local data = Network._channel.receive ()
    if data then
        local msgType, msgArgs 
        local i = string.find ( data, "\n" )
        if i then
            msgType = string.sub ( data, 1, i - 1 )
            msgArgs = MOAIJsonParser.decode ( string.sub (  data, i + 1 ))
            app.queue:postEvent ( nil, NetEvents [ msgType ], msgArgs )
        end
    end
end

return Network
