local resources = {
    
    textureCache = ResourceCache.new (
        function ( key )
            local texture = MOAITexture.new ()
            texture:load ( key )
            texture:setFilter ( MOAITexture.GL_LINEAR, MOAITexture.GL_LINEAR )
            return texture
        end
    ),
    
    fontCache = ResourceCache.new (
        function ( key )
            local font = MOAIFont.new ()
            if key:match( ".+%.fnt" ) then
                font:loadFromBMFont ( key )
            else
                font:load ( key )
            end
            return font
        end
    ),
    
    soundCache = ResourceCache.new (
        function ( key )
            local sound = MOAIUntzSound.new ()
            sound:load ( key )
            return sound
        end
    ),
}

function resources.processTPScript ( fileName, pathToTextures )
    local rsrc = dofile ( fileName )
    local deck = MOAIGfxQuadDeck2D.new ()
    local pathToTextures = pathToTextures or ""
    
    
    local texture = MOAITexture.new ()
    texture:load ( pathToTextures .. rsrc.texture )
    texture:setFilter ( MOAITexture.GL_LINEAR, MOAITexture.GL_LINEAR )
    deck:setTexture ( texture )
    
    local frames = rsrc.frames
    local scaleFactor = rsrc.scaleFactor or 1.0
    deck:reserve ( #frames )
    for idx, frame in ipairs ( frames ) do
        local rect = frame.spriteColorRect
        local spriteSize = frame.spriteSourceSize
        deck:setRect ( idx, scaleFactor * rect.x, scaleFactor * ( spriteSize.height - rect.y - rect.height ), scaleFactor * ( rect.x + rect.width ), scaleFactor * ( spriteSize.height - rect.y ))
        local uvRect = frame.uvRect
        if frame.textureRotated then
            deck:setUVQuad ( idx, uvRect.u1, uvRect.v0, uvRect.u1, uvRect.v1, uvRect.u0, uvRect.v1, uvRect.u0, uvRect.v0 )
        else
            deck:setUVQuad ( idx, uvRect.u0, uvRect.v0, uvRect.u1, uvRect.v0, uvRect.u1, uvRect.v1, uvRect.u0, uvRect.v1 )
        end
    
    end
    
    return deck
end
    
return resources