local core = require ( "core" )

local BoardView = core.class ( Object )

local BoardModel = require ( "Model.BoardModel" )

function BoardView:ctor ( size )
    local size = size or 256
    local scriptDeck = MOAIScriptDeck.new ()
    scriptDeck:setRect ( -size, -size, size, size )
    
    scriptDeck:setDrawCallback ( 
        function ( index, xOff, yOff, xFlip, yFlip )
            local boardModel = self._boardModel
            if boardModel then
                local circles = boardModel:circles ()
                local sectors = boardModel:sectors ()
                
                for k, circle in pairs ( circles ) do
                    MOAIDraw.drawCircle ( 0, 0, circle.r, BoardModel.STEPS )
                end
                
                local begR = circles [ BoardModel.R_BULL ].r
                local endR = circles [ BoardModel.R_DOUBLE_RING ].r
                for k, sector in pairs ( sectors ) do
                    local bx, by = boardModel.modelCoordinates ( begR, sector.f )
                    local ex, ey = boardModel.modelCoordinates ( endR, sector.f )
                    MOAIDraw.drawLine ( bx, by, ex, ey )
                end
                
                if boardModel.lastTarget then
                    local x, y = unpack ( boardModel.lastTarget )
                    MOAIDraw.fillCircle ( x, y, 1 )
                end
            end
        end
        )

    local prop = MOAIProp2D.new ()
    prop:setDeck ( scriptDeck )
    self._prop = prop
end

function BoardView:setBoardModel ( boardModel )
    self._boardModel =  boardModel
end

function BoardView:boardModel ()
    return self._boardModel
end

function BoardView:show ( layer, x, y )
    if layer then
        layer:insertProp ( self._prop )
        self._prop:setLoc ( x, y + 1 )
        self._layer = layer
    end
end

function BoardView:hide ()
    if self._layer then
        self._layer:removeProp ( self._prop )
        self._layer = nil
    end
end

return BoardView
