local core = require ( "core" )
local Events = require ( "Events" )
local SoundManager = require ( "SoundManager" )
local Actions = Loaders.Actions
local AnimEvents = Events.E_ANIMATION
local GameEvents = Events.E_GAME

local function indexAnimation ( target, from, to, length )
    local n = to - from + 1
    
    local curve = MOAIAnimCurve.new ()
    curve:reserveKeys ( n + 1 )
    
    local idx = from
    local t = 0.0
    local dt = length / n
    for i = 1, n do
        curve:setKey (i, t, idx, MOAIEaseType.FLAT)
        t = t + dt
        idx = idx + 1 
    end
    
    curve:setKey ( n + 1, length, n, MOAIEaseType.FLAT )

    local anim = MOAIAnim:new ()
    anim:reserveLinks ( 1 )
    anim:setLink ( 1, curve, target, MOAIProp2D.ATTR_INDEX )
    return anim
end

local function postEvent ( event )
    return 
        function ( target )
            app.queue:postEvent ( target, event )
        end
end

local function moveTo ( nodeName, x, y, length, easeMode )
    return 
        function ( target )
            return target [ nodeName ]:seekLoc ( x, y, length, easeMode )
        end
end

local function moveBy ( nodeName, x, y, length, easeMode )
    return 
        function ( target )
            return target [ nodeName ]:moveLoc ( x, y, length, easeMode )
        end
end

local function rotateBy ( nodeName, ang, length )
    return 
        function ( target )
            return target [ nodeName ]:moveRot ( ang, length )
        end
end

local function tintTo ( nodeName, r, g, b, a, length )
    return 
        function ( target )
            return target [ nodeName ]:seekColor ( r, g, b, a, length )
        end
end

local actions = 
{
    ThrowDarts = rm:rsrc ( Actions.Chain )
    {
        rm:rsrc ( Actions.Split )
        {
            function ( target, options )
                return target:moveLoc ( options.dx, options.dy, options.length )
            end;
            
            function ( target, options ) 
                return indexAnimation ( target, 1, 11, options.length )
            end;
        };
        
        onDone = postEvent ( AnimEvents.DARTS_ANIM );
    };
    
    EndTurnAnim = rm:rsrc ( Actions.Chain )
    {
        rm:rsrc ( Actions.Wait )
        {
            t = 1.3
        };
        
        onDone = postEvent ( GameEvents.SWITCH_TURN );
    };
    
    SplashAnim = rm:rsrc ( Actions.Chain )
    {
        
        rm:rsrc ( Actions.Split )
        {
            tintTo ( "dartsLogo", 1, 1, 1, 1, 1.6 );
            rotateBy ( "dartsLogo", -1080, 1.6 );
            
            rm:rsrc ( Actions.Timeline )
            {
                {
                    t = 0.2,
                    function ( self )
                        SoundManager.play ( "splash.ogg" )
                    end
                },
            };
        };
        
        rm:rsrc ( Actions.Wait )
        {
            t = 1.0
        };
        
        rm:rsrc ( Actions.Split )
        {
            moveTo ( "bottomPanel", 0, 0, 1.0 );
            moveTo ( "topPanel", 0, RealViewSize.height, 1.0 );
            moveTo ( "splashScreen", RealViewSize.width, 0, 1.0 );

            tintTo ( "moaiLogo", 1, 1, 1, 0, 0.25 );
        };
        
        onDone = postEvent ( E_SYSTEM.LEAVE_SPLASH );
    };
    
    FadeColor = rm:rsrc ( Actions.Chain )
    {
        function ( target, options ) 
            return target:seekColor ( options.r, options.g, options.b , options.a, options.length )
        end;        
    };
    
    HintSwipe = rm:rsrc ( Actions.Chain )
    {
        rm:rsrc ( Actions.Wait )
        {
            t = 1.0
        };
        
        tintTo ( "hintSwipe", 1, 1, 1, 1, 0.1, MOAIEaseType.FLAT );
        rm:rsrc ( Actions.Wait )
        {
            t = 0.25
        };
        moveBy ( "hintSwipe", 0, 150, 0.7, MOAIEaseType.EASE_OUT );
        
        onDone = function ( target )
            target.hintSwipe:setColor ( 1, 1, 1, 0, 0.1 );
            target.hintSwipe:addLoc ( 0, -150 )
            app.queue:postEvent ( target, AnimEvents.SWIPE_HINT )
        end;
    };
}

return actions