local core = require ( "core" )
local Events = require ( "Events" )
local Resources = require ( "Resources" )
local SoundManager = require ( "SoundManager" )

local AnimEvents = Events.E_ANIMATION

local dartsTargetNumbersDeck = Resources.processTPScript ( "Resources/Scripts/dart_target_numbers.lua", "Resources/Images/" )
local dartsSplashLettersDeck = Resources.processTPScript ( "Resources/Scripts/splash_letters.lua", "Resources/Images/" )
local sectorHighilghtsDeck = Resources.processTPScript ( "Resources/Scripts/sector_highlights.lua", "Resources/Images/" )

local alphaBlendMode = { MOAIProp.GL_SRC_ALPHA, MOAIProp.GL_ONE_MINUS_SRC_ALPHA }
local defaulFont = "Resources/Fonts/trade_gothic.fnt"

local scoreViewOffset = ( RealViewSize.width - 274 ) * 0.25


local function onTouchEnter ( self )
    self:setScl ( 1.1 )
end

local function onTouchLeave ( self )
    self:setScl ( 1.0 )
end

local resources = 
{
    rm:rsrc ( Loaders.Layer, "gameBoardLayer" ) { };
    rm:rsrc ( Loaders.Layer, "splashScreenLayer" ) { };
    
    rm:rsrc ( Loaders.Layer, "gameDartsLayer" )
    {
        sortMode = MOAILayer.SORT_Y_DESCENDING
    };
    
    rm:rsrc ( Loaders.Layer, "playerHUDLayer" )
    {
        partition = rm:rsrc ( Loaders.Partition )
        {
            levels = {{ 256, 4, 4 }, { 28, 8, 8 }};
            panAllowed = true;
        };
    };

    rm:rsrc ( Loaders.Layer, "popupLayer" )
    {
        partition = rm:rsrc ( Loaders.Partition )
        {
            levels = {{ 256, 4, 4 }, { 28, 8, 8 }};
        };
        sortMode = MOAILayer.SORT_PRIORITY_ASCENDING,
    };
    
    rm:rsrc ( Loaders.GfxQuadDeck, "dartsTargetDeck" ) 
    { 
        reserve = 4,
        texture = "Resources/Images/darts_target_q.png",
        rects = {{ 0, 0, 150, 151 }; { 0, 0, -150, 151 }; { 0, 0, -150, -151 }; { 0, 0, 150, -151 }},
        UVRects = {{ 0, 1, 1, 0 }; { 0, 1, 1, 0 }; { 0, 1, 1, 0 }; { 0, 1, 1, 0 }},
    };
    
    rm:rsrc ( Loaders.GfxQuad, "dartsHighlightDeck" ) 
    {
        texture = "Resources/Images/darts_highlight.png",
        rect = { 123.5, 0, 207.5, 147 },
    };
    
    rm:rsrc ( Loaders.TileDeck, "dartsCountersDeck" ) 
    { 
        texture = "Resources/Images/darts_counters.png",
        rect = { 0, 0, 31, 127.5 },
        size = { 2, 1 },
    };
    
    rm:rsrc ( Loaders.Base, "dartsDecks" )
    {
        Resources.processTPScript ( "Resources/Scripts/dart_fly_b.lua", "Resources/Images/" ),
        Resources.processTPScript ( "Resources/Scripts/dart_fly_r.lua", "Resources/Images/" ),
    };
    
    rm:rsrc ( Loaders.Transform, "inputRoot" )
    {
        rm:rsrc ( Loaders.Transform, "mainRoot" )
        {
            rm:rsrc ( Loaders.Transform, "popupElements" )
            {
                rm:rsrc ( Loaders.Prop, "hintSwipe" )
                {
                    layer = "popupLayer",
                    loc = { RealViewSize.width * 0.5, 80 },
                    blendMode = alphaBlendMode,
                    color = { 1, 1, 1, 0 },
                    visible = false,
                    deck  = rm:rsrc ( Loaders.GfxQuad )
                    {
                        texture = "Resources/Images/darts_hints_hand.png",
                        rect = { -47.5, -67.5, 0, 0 },
                    };
                };
                rm:rsrc ( Loaders.Prop, "fader" )
                {
                    layer = "popupLayer",
                    deck  = rm:rsrc ( Loaders.GfxQuad )
                    {
                        texture = "Resources/Images/darts_fader.png",
                        rect = { 0, 0, RealViewSize.width, RealViewSize.height },
                    };
                    color = { 0, 0, 0, 0 },
                };
                
                rm:rsrc ( Loaders.TextBox, "hintText" )
                {
                    layer = "popupLayer",
                    rect = { -RealViewSize.width * 0.5, -100, RealViewSize.width * 0.5, 100 },
                    alignment = { MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY },
                    font = defaulFont,
                    textSize = 35,
                    blendMode = alphaBlendMode,
                    color = { 1, 1, 1, 0 },
                    loc = { RealViewSize.width * 0.5, RealViewSize.height * 0.7 },
                    yFlip = true,
                };
                
                rm:rsrc ( Loaders.TextBox, "turnLabel" )
                {
                    layer = "popupLayer",
                    rect = { -RealViewSize.width * 0.5 + 5, 0, RealViewSize.width * 0.5 - 5, 100 },
                    alignment = { MOAITextBox.CENTER_JUSTIFY, MOAITextBox.RIGHT_JUSTIFY },
                    font = defaulFont,
                    textSize = 20,
                    blendMode = alphaBlendMode,
                    color = { 1, 1, 1, 0.75 },
                    loc = { RealViewSize.width * 0.5, 170 },
                    yFlip = true,
                };
            };
            
            rm:rsrc ( Loaders.Transform, "gameBg" )
            {
                loc     =   { 0, 0 },
                rm:rsrc ( Loaders.Prop )
                {
                    layer = "gameBoardLayer",
                    deck  = rm:rsrc ( Loaders.GfxQuad )
                    {
                        texture = rm:rsrc ( Loaders.Texture )
                        {
                            source = "Resources/Images/darts_bg.jpg",
                            wrap = true,
                        },
                        rect = { 0, 0, RealViewSize.width, RealViewSize.height },
                        UVRect = { 0, RealViewSize.height / 128, RealViewSize.width / 128, 0 },
                    };
                    loc = { 0, 0 },
                };
            };
            
            rm:rsrc ( Loaders.Transform, "splashScreen" )
            {
                rm:rsrc ( Loaders.Prop, "dartsLogo" )
                {
                    layer = "splashScreenLayer",
                    deck  = rm:rsrc ( Loaders.GfxQuad )
                    {
                        texture = "Resources/Images/darts_logo.png",
                        rect = { -119, -83.0, 119, 83.0 },
                    };
                    loc = { RealViewSize.width * 0.5, RealViewSize.height * 0.6 },
                    blendMode = alphaBlendMode,
                    color = { 1, 1, 1, 0 },
                    rm:rsrc ( Loaders.Prop )
                    {
                        layer = "splashScreenLayer",
                        deck  = rm:rsrc ( Loaders.GfxQuad )
                        {
                            texture = "Resources/Images/darts_logo_301.png",
                            rect = { 0, 0, 64, 38 },
                        };
                        blendMode = alphaBlendMode,
                        loc = { 63, 13 },
                    };
                };

                rm:rsrc ( Loaders.Prop, "moaiLogo" )
                {
                    layer = "splashScreenLayer",
                    deck  = rm:rsrc ( Loaders.GfxQuad )
                    {
                        texture = "Resources/Images/moai_logo.png",
                        rect = { -149, 0, 0, 46 },
                    };

                    loc = { RealViewSize.width - 15, 15 },
                },
            };
            
            rm:rsrc ( Loaders.Transform, "playerHUD" )
            {
                loc     =   { 0, 0 },
                rm:rsrc ( Loaders.Transform, "topPanel" )
                {
                    loc = { 0, RealViewSize.height + 100 },
                    rm:rsrc ( Loaders.Prop )
                    {
                        layer = "playerHUDLayer",
                        deck  = rm:rsrc ( Loaders.GfxQuad )
                        {
                            texture = "Resources/Images/darts_panel_bottom.png",
                            rect = { 0, -49.5, RealViewSize.width, 50.0 },
                            UVRect = { 0, 0.5, 1, 1.0 },
                        };
                    };
                    rm:rsrc ( Loaders.Prop )
                    {
                        layer = "gameBoardLayer",
                        deck  = rm:rsrc ( Loaders.GfxQuad )
                        {
                            texture = "Resources/Images/darts_shad_bg.png",
                            rect = { 0, -153, 153, 0 },
                            UVRect = { 0, 1, 1, 0 },
                        };
                        loc = { 0, -49.5 }
                    };
                    rm:rsrc ( Loaders.Prop )
                    {
                        layer = "gameBoardLayer",
                        deck  = rm:rsrc ( Loaders.GfxQuad )
                        {
                            texture = "Resources/Images/darts_shad_bg.png",
                            rect = { -153, -153, 0, 0 },
                            UVRect = { 1, 1, 0, 0 },
                        };
                        loc = { RealViewSize.width, -49.5 }
                    };
                    rm:rsrc ( Loaders.Prop )
                    {
                        layer = "playerHUDLayer",
                        deck  = rm:rsrc ( Loaders.GfxQuad )
                        {
                            texture = "Resources/Images/darts_info.png",
                            rect = { 0, 0, 22.5, 22.5 },
                            bounds = {-11, -11, 33, 33},
                        };
                        loc = { 12.0, -35 },

                        isClickable = true;
                        onClick = function ( self )
                            SoundManager.play ( "click.ogg" )
                            app.queue:postEvent ( self, Events.E_GAME.INFO, nil )
                        end;
                        onTouchEnter = onTouchEnter;
                        onTouchLeave = onTouchLeave;
                    };
                    rm:rsrc ( Loaders.Prop, "exitButton" )
                    {
                        layer = "playerHUDLayer",
                        deck  = rm:rsrc ( Loaders.GfxQuad )
                        {
                            texture = "Resources/Images/darts_exit.png",
                            rect = { 0, 0, 22.5, 22.5 },
                            bounds = {-11, -11, 33, 33},
                        };
                        loc = { RealViewSize.width - 33, -35 },

                        isClickable = true;
                        onClick = function ( self )
                            SoundManager.play ( "click.ogg" )
                            app.queue:postEvent ( self, Events.E_GAME.QUIT_PROMPT, nil )
                        end;
                        onTouchEnter = onTouchEnter;
                        onTouchLeave = onTouchLeave;
                    };
                    rm:rsrc ( Loaders.Transform )
                    {
                        loc = { 32 + scoreViewOffset, -42 },
                        rm:rsrc ( Loaders.Prop )
                        {
                            layer = "playerHUDLayer",
                            deck  = rm:rsrc ( Loaders.GfxQuad )
                            {
                                texture = "Resources/Images/darts_screen.png",
                                rect = { 0, 0, 70, 35 },
                            };
                        };
                        rm:rsrc ( Loaders.TextBox, "scoreBox_1" )
                        {
                            layer = "playerHUDLayer",
                            rect = { -35, -17.5, 35, 17.5 },
                            alignment = { MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY },
                            font = defaulFont,
                            textSize = 20,
                            loc = { 35, 17.5 },
                            yFlip = true,
                        };
                    };
                    rm:rsrc ( Loaders.Transform )
                    {
                        loc = { 32 + 70 + scoreViewOffset * 2, -42 },
                        rm:rsrc ( Loaders.Prop )
                        {
                            layer = "playerHUDLayer",
                            deck  = rm:rsrc ( Loaders.GfxQuad )
                            {
                                texture = "Resources/Images/darts_screen.png",
                                rect = { 0, 0, 70, 35 },
                            };
                        };
                        rm:rsrc ( Loaders.TextBox, "scoreBox_2" )
                        {
                            layer = "playerHUDLayer",
                            rect = { -35, -17.5, 35, 17.5 },
                            alignment = { MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY },
                            font = defaulFont,
                            textSize = 20,
                            loc = { 35, 17.5 },
                            yFlip = true,
                        };
                    };
                    rm:rsrc ( Loaders.Transform )
                    {
                        loc = { 32 + 140 + scoreViewOffset * 3, -42 },
                        rm:rsrc ( Loaders.Prop )
                        {
                            layer = "playerHUDLayer",
                            deck  = rm:rsrc ( Loaders.GfxQuad )
                            {
                                texture = "Resources/Images/darts_screen.png",
                                rect = { 0, 0, 70, 35 },
                            };
                        };
                        rm:rsrc ( Loaders.TextBox, "scoreBox_3" )
                        {
                            layer = "playerHUDLayer",
                            rect = { -35, -17.5, 35, 17.5 },
                            alignment = { MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY },
                            font = defaulFont,
                            textSize = 20,
                            loc = { 35, 17.5 },
                            yFlip = true,
                        };
                    };
                };
                rm:rsrc ( Loaders.Transform, "bottomPanel" )
                {
                    loc = { 0, -100 },
                    rm:rsrc ( Loaders.Prop )
                    {
                        layer = "playerHUDLayer",
                        deck  = rm:rsrc ( Loaders.GfxQuad )
                        {
                            texture = "Resources/Images/darts_panel_bottom.png",
                            rect = { 0, 0, RealViewSize.width, 200 },
                        };
                    };
                    rm:rsrc ( Loaders.Prop )
                    {
                        layer = "gameBoardLayer",
                        deck  = rm:rsrc ( Loaders.GfxQuad )
                        {
                            texture = "Resources/Images/darts_shad_bg.png",
                            rect = { 0, 0, 153, 153 },
                            UVRect = { 0, 0, 1, 1 },
                        };
                        loc = { 0, 99.5 }
                    };
                    rm:rsrc ( Loaders.Prop )
                    {
                        layer = "gameBoardLayer",
                        deck  = rm:rsrc ( Loaders.GfxQuad )
                        {
                            texture = "Resources/Images/darts_shad_bg.png",
                            rect = { -153, 0, 0, 153 },
                            UVRect = { 1, 0, 0, 1 },
                        };
                        loc = { RealViewSize.width, 99.5 }
                    };
                    rm:rsrc ( Loaders.Prop, "dartsCounter_11" )
                    {
                        layer = "playerHUDLayer",
                        deck  = "dartsCountersDeck",
                        loc = { 0, 0 },
                        index = 1,
                        visible = false,
                    };
                    rm:rsrc ( Loaders.Prop, "dartsCounter_12" )
                    {
                        layer = "playerHUDLayer",
                        deck  = "dartsCountersDeck",
                        loc = { 15, 0 },
                        index = 1,
                        visible = false,
                    };
                    rm:rsrc ( Loaders.Prop, "dartsCounter_13" )
                    {
                        layer = "playerHUDLayer",
                        deck  = "dartsCountersDeck",
                        loc = { 30, 0 },
                        index = 1,
                        visible = false,
                    };
                    rm:rsrc ( Loaders.Prop, "dartsCounter_21" )
                    {
                        layer = "playerHUDLayer",
                        deck  = "dartsCountersDeck",
                        loc = { RealViewSize.width, 0 },
                        piv = { 31, 0 },
                        index = 2,
                        visible = false,
                    };
                    rm:rsrc ( Loaders.Prop, "dartsCounter_22" )
                    {
                        layer = "playerHUDLayer",
                        deck  = "dartsCountersDeck",
                        loc = { RealViewSize.width - 15, 0 },
                        piv = { 31, 0 },
                        index = 2,
                        visible = false,
                    };
                    rm:rsrc ( Loaders.Prop, "dartsCounter_23" )
                    {
                        layer = "playerHUDLayer",
                        deck  = "dartsCountersDeck",
                        loc = { RealViewSize.width - 30, 0 },
                        piv = { 31, 0 },
                        index = 2,
                        visible = false,
                    };
                    rm:rsrc ( Loaders.Transform )
                    {
                        loc = { ( RealViewSize.width - 211 ) * 0.5, 0 },
                        rm:rsrc ( Loaders.Prop )
                        {
                            layer = "playerHUDLayer",
                            deck  = rm:rsrc ( Loaders.GfxQuad )
                            {
                                texture = "Resources/Images/darts_screen.png",
                                rect = { 0, 0, 211, 95 },
                            };
                        };
                        rm:rsrc ( Loaders.Prop )
                        {
                            loc = { 100, 10 },
                            layer = "playerHUDLayer",
                            deck  = rm:rsrc ( Loaders.GfxQuad )
                            {
                                texture = "Resources/Images/darts_screen_devider.png",
                                rect = { 0, 0, 11, 78 },
                            };
                        };
                        rm:rsrc ( Loaders.TextBox, "nameBox_1" )
                        {
                            layer = "playerHUDLayer",
                            rect = { -128, -128, 128, 128 },
                            alignment = { MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY },
                            font = defaulFont,
                            textSize = 20,
                            loc = { 63, 65 },
                            yFlip = true,
                        };
                        rm:rsrc ( Loaders.TextBox, "nameBox_2" )
                        {
                            layer = "playerHUDLayer",
                            rect = { -128, -128, 128, 128 },
                            alignment = { MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY },
                            font = defaulFont,
                            textSize = 20,
                            loc = { 145, 65 },
                            yFlip = true,
                        };
                        rm:rsrc ( Loaders.TextBox, "textBox_1" )
                        {
                            layer = "playerHUDLayer",
                            rect = { -128, -128, 128, 128 },
                            alignment = { MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY },
                            font = defaulFont,
                            textSize = 35,
                            loc = { 63, 35 },
                            yFlip = true,
                        };
                        rm:rsrc ( Loaders.TextBox, "textBox_2" )
                        {
                            layer = "playerHUDLayer",
                            rect = { -128, -128, 128, 128 },
                            alignment = { MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY },
                            font = defaulFont,
                            textSize = 35,
                            loc = { 145, 35 },
                            yFlip = true,
                        };
                        rm:rsrc ( Loaders.Prop, "turnIndicator_1" )
                        {
                            layer = "playerHUDLayer",
                            deck  = rm:rsrc ( Loaders.GfxQuadDeck )
                            {
                                reserve = 2,
                                texture = "Resources/Images/darts_indicators.png",
                                rects = {{ 0, 0, 10.5, 39.5 }; { 0, 0, 10.5, 39.5 }},
                                UVRects = {{ 0, 0, 1/3, 1 }; { 2/3, 0, 1, 1 }},
                            };
                            loc = { 15.5, 30.0 },
                            index = 2,
                        };
                        rm:rsrc ( Loaders.Prop, "turnIndicator_2" )
                        {
                            layer = "playerHUDLayer",
                            deck  = rm:rsrc ( Loaders.GfxQuadDeck )
                            {
                                reserve = 2,
                                texture = "Resources/Images/darts_indicators.png",
                                rects = {{ 0, 0, 10.5, 39.5 }; { 0, 0, 10.5, 39.5 }},
                                UVRects = {{ 1/3, 0, 2/3, 1 }; { 2/3, 0, 1, 1 }},
                            };
                            loc = { 185.5, 30.0 },
                            index = 2,
                        };
                    };
                };
            };

            rm:rsrc ( Loaders.Transform, "gameBoard" )
            {
                loc = { RealViewSize.width * 0.5, RealViewSize.height * 0.55 },
                rm:rsrc ( Loaders.Prop, "gameBoardQ" )
                {
                    layer = "gameBoardLayer",
                    deck = "dartsTargetDeck",
                    index = 1,
                    loc = { 0, 1 },
                };
                rm:rsrc ( Loaders.Prop )
                {
                    layer = "gameBoardLayer",
                    deck = "dartsTargetDeck",
                    index = 2,
                    loc = { 0, 1 },
                };
                rm:rsrc ( Loaders.Prop )
                {
                    layer = "gameBoardLayer",
                    deck = "dartsTargetDeck",
                    index = 3,
                    loc = { 0, 1 },
                };
                rm:rsrc ( Loaders.Prop )
                {
                    layer = "gameBoardLayer",
                    deck = "dartsTargetDeck",
                    index = 4,
                    loc = { 0, 1 },
                };
                rm:rsrc ( Loaders.Prop )
                {
                    layer = "gameBoardLayer",
                    deck = rm:rsrc ( Loaders.GfxQuad )
                    {
                        texture = "Resources/Images/darts_target_shadow.png",
                        rect = { 0, 0, 250, 110 },
                    };
                    loc = { -142.5, -154 },
                };
                rm:rsrc ( Loaders.Prop, "sectorHighlight" )
                {
                    layer = "gameBoardLayer",
                    deck = sectorHighilghtsDeck,
                    loc = { 0, 1 },
                    piv = { 28.1, 22 },
                    visible = false,
                };
                
                rm:rsrc ( Loaders.Transform )
                {
                    loc = { -159.5, -284 },
                    rm:rsrc ( Loaders.Prop )
                    {
                        layer = "gameBoardLayer",
                        deck = dartsTargetNumbersDeck,
                        index = 1,
                    };
                    rm:rsrc ( Loaders.Prop )
                    {
                        layer = "gameBoardLayer",
                        deck = dartsTargetNumbersDeck,
                        index = 2,
                    };
                    rm:rsrc ( Loaders.Prop )
                    {
                        layer = "gameBoardLayer",
                        deck = dartsTargetNumbersDeck,
                        index = 3,
                    };
                    rm:rsrc ( Loaders.Prop )
                    {
                        layer = "gameBoardLayer",
                        deck = dartsTargetNumbersDeck,
                        index = 4,
                    };
                    rm:rsrc ( Loaders.Prop )
                    {
                        layer = "gameBoardLayer",
                        deck = dartsTargetNumbersDeck,
                        index = 5,
                    };
                    rm:rsrc ( Loaders.Prop )
                    {
                        layer = "gameBoardLayer",
                        deck = dartsTargetNumbersDeck,
                        index = 6,
                    };
                    rm:rsrc ( Loaders.Prop )
                    {
                        layer = "gameBoardLayer",
                        deck = dartsTargetNumbersDeck,
                        index = 7,
                    };
                    rm:rsrc ( Loaders.Prop )
                    {
                        layer = "gameBoardLayer",
                        deck = dartsTargetNumbersDeck,
                        index = 8,
                    };
                    rm:rsrc ( Loaders.Prop )
                    {
                        layer = "gameBoardLayer",
                        deck = dartsTargetNumbersDeck,
                        index = 9,
                    };
                    rm:rsrc ( Loaders.Prop )
                    {
                        layer = "gameBoardLayer",
                        deck = dartsTargetNumbersDeck,
                        index = 10,
                    };
                    rm:rsrc ( Loaders.Prop )
                    {
                        layer = "gameBoardLayer",
                        deck = dartsTargetNumbersDeck,
                        index = 11,
                    };
                    rm:rsrc ( Loaders.Prop )
                    {
                        layer = "gameBoardLayer",
                        deck = dartsTargetNumbersDeck,
                        index = 12,
                    };
                    rm:rsrc ( Loaders.Prop )
                    {
                        layer = "gameBoardLayer",
                        deck = dartsTargetNumbersDeck,
                        index = 13,
                    };
                    rm:rsrc ( Loaders.Prop )
                    {
                        layer = "gameBoardLayer",
                        deck = dartsTargetNumbersDeck,
                        index = 14,
                    };
                    rm:rsrc ( Loaders.Prop )
                    {
                        layer = "gameBoardLayer",
                        deck = dartsTargetNumbersDeck,
                        index = 15,
                    };
                    rm:rsrc ( Loaders.Prop )
                    {
                        layer = "gameBoardLayer",
                        deck = dartsTargetNumbersDeck,
                        index = 16,
                    };
                    rm:rsrc ( Loaders.Prop )
                    {
                        layer = "gameBoardLayer",
                        deck = dartsTargetNumbersDeck,
                        index = 17,
                    };
                    rm:rsrc ( Loaders.Prop )
                    {
                        layer = "gameBoardLayer",
                        deck = dartsTargetNumbersDeck,
                        index = 18,
                    };
                    rm:rsrc ( Loaders.Prop )
                    {
                        layer = "gameBoardLayer",
                        deck = dartsTargetNumbersDeck,
                        index = 19,
                    };
                    rm:rsrc ( Loaders.Prop )
                    {
                        layer = "gameBoardLayer",
                        deck = dartsTargetNumbersDeck,
                        index = 20,
                    };
                };
            };
        };
    };
}

return resources
