local core = require ( "core" )
local Resources = require ( "Resources" )

local alphaBlendMode = { MOAIProp.GL_SRC_ALPHA, MOAIProp.GL_ONE_MINUS_SRC_ALPHA }
local addBlendMode = { MOAIProp.GL_ONE, MOAIProp.GL_ONE }
local defaulFont = "Resources/Fonts/trade_gothic.fnt"

local textBox = function ( label, string, y )
    return rm:rsrc ( Loaders.TextBox, label )
    {
        layer = "mainLayer",
        rect = { -160, 0, 160, 100.0 },
        alignment = { MOAITextBox.CENTER_JUSTIFY, MOAITextBox.LEFT_JUSTIFY },
        font = defaulFont,
        textSize = 15,
        yFlip = true,
        string = string,
        loc = { RealViewSize.width * 0.5, y }
    }
end

local lighter = function ()
    return rm:rsrc ( Loaders.Prop )
    {
        layer = "mainLayer",
        deck = "lighterDeck",
        color = { 0.25, 0.25, 0.25, 1.0 }, 
        blendMode = addBlendMode,
    };
end

local hintText = function ( string, right, x, y )
    return rm:rsrc ( Loaders.TextBox )
    {
        layer = "mainLayer",
        rect = { right and 0 or -100, 0, right and 100 or 0, 20.0 },
        alignment = { right and MOAITextBox.LEFT_JUSTIFY or MOAITextBox.RIGHT_JUSTIFY, MOAITextBox.RIGHT_JUSTIFY },
        font = defaulFont,
        textSize = 15,
        yFlip = true,
        string = string,
        loc = {( right and 1 or -1 ) * 56 + ( x or 0 ), -3 + ( y or 0 )}
    }
end

local resources = 
{
    rm:rsrc ( Loaders.Layer, "mainLayer" )
    {
    };
    
    rm:rsrc ( Loaders.GfxQuadDeck, "arrowsDeck" )
    {
        reserve = 4,
        texture = "Resources/Images/darts_hints_arrow.png",
        rects = {{ -3, -3, 53, 10.5 }; { -53, -3, 3, 10.5 }; { -53, -10.5, 3, 3 }; { -3, -10.5, 53, 3 }},
        UVRects = {{ 0, 1, 1, 0 }; { 1, 1, 0, 0 }; { 1, 0, 0, 1 }; { 0, 0, 1, 1 }},
    };
    
    rm:rsrc ( Loaders.GfxQuad, "lighterDeck" )
    {
        texture = "Resources/Images/darts_hints_lighter.png",
        rect = { -23.5, -23.5, 23.5, 23.5 },
    };
    
    rm:rsrc ( Loaders.Transform, "root" )
    {
        rm:rsrc ( Loaders.Transform )
        {
            loc = { RealViewSize.width * 0.5, RealViewSize.height * 0.55 + 1 },
            rm:rsrc ( Loaders.Prop )
            {
                layer = "mainLayer",
                deck = "arrowsDeck",
                index = 4,
                lighter (),
                hintText ( "Bullseye - 50", true ),
            };
            rm:rsrc ( Loaders.Transform )
            {
                loc = { -8, 0 },
                hintText ( "Bull - 25", false, 10, 26 ),
                rm:rsrc ( Loaders.Prop )
                {
                    layer = "mainLayer",
                    deck = "arrowsDeck",
                    rot = -30,
                    index = 2,
                };
            };
            rm:rsrc ( Loaders.Prop )
            {
                loc = { -110, -44 },
                layer = "mainLayer",
                deck = "arrowsDeck",
                lighter (),
                hintText ( "Score value x2", true ),
            };
            rm:rsrc ( Loaders.Prop )
            {
                loc = { -23, 70 },
                layer = "mainLayer",
                deck = "arrowsDeck",
                lighter (),
                index = 3,
                hintText ( "Score value x3", false ),
            };
            rm:rsrc ( Loaders.Transform )
            {
                loc = { -0, 50 },
                hintText ( "Score value", true, -10, 26 ),
                rm:rsrc ( Loaders.Prop )
                {
                    layer = "mainLayer",
                    deck = "arrowsDeck",
                    rot = 30,
                    lighter (),
                };
            };
        };
            
        rm:rsrc ( Loaders.Prop )
        {
            textBox ( "label_1", "Both players begin with 301 points and try to reach zero first", RealViewSize.height - 160 );
            textBox ( "label_2", "Each round, the sum of three darts will be withdrawn \nfrom the remaining score", RealViewSize.height - 160 - 20 );
            textBox ( "label_3", "The first player to reach exactly zero points is the winner", 60 + 20 );
            textBox ( "label_4", "Players getting a negative score will lose all points \nscored in that round", 60 );
        };
        
    };
}

return resources
