local _NAME = ...
local core = require ( "core" )

local Events = require ( "Events" )
local AnimEvents = Events.E_ANIMATION

local MessageBox = core.class ( Object )
{
    BUTTON_LEFT = 1,
    BUTTON_RIGHT = 2,
};

function MessageBox:ctor ( layer, priority )
    local resource = require ( _NAME .. ".resources" )
    for _, res in pairs ( resource ) do
        res:create ( self )
    end
    
    self._buttons = { self:_extractButton ( MessageBox.BUTTON_LEFT ), self:_extractButton ( MessageBox.BUTTON_RIGHT )}
    self._layer = layer
    if priority then
        self.backGroundShadow:setPriority ( priority )
        self.backGround:setPriority ( priority + 1 )
        self.titleTextBox:setPriority ( priority + 2 )
        self.infoTextBox:setPriority ( priority + 2 )
        for _, bt in ipairs ( self._buttons ) do
            bt.shadow:setPriority ( priority + 2 )
            bt.bg:setPriority ( priority + 3 )
            bt.text:setPriority ( priority + 4 )
        end
    end
end

function MessageBox:_extractButton ( button )
   return { bg = self [ "button_" ..  button ], text = self [ "butttonT_" ..  button ], shadow = self [ "buttonShadow_" ..  button ]}
end

function MessageBox:setText ( text )
    self.titleTextBox:setString ( text )
end

function MessageBox:setInfoText ( text )
    self.infoTextBox:setString ( text )
    self._infoString = text
end

function MessageBox:setButtonText ( button, text )
    self._buttons [ button ].text:setString ( text )
end

function MessageBox:setButtonEvent ( button, event )
    self._buttons [ button ].bg.tapEvent = event
end

function MessageBox:setIndex ( index )
    self.backGround:setIndex ( index )
    self.button_1:setIndex ( index )
    self.button_2:setIndex ( index )
end

function MessageBox:show ( showButtons, animated )
    if self._hideAction then
        self._hideAction:stop ()
    end

    assert ( not self._visible )
    self._visible = true
    local layer = self._layer
    
    layer:insertProp ( self.backGroundShadow )
    layer:insertProp ( self.backGround )
    layer:insertProp ( self.titleTextBox )
    self._titleShift = false
    if showButtons then
        self.titleTextBox:addLoc ( 0, 20 )
        self._titleShift = true
        for _, bt in ipairs ( self._buttons ) do
            layer:insertProp ( bt.shadow )
            layer:insertProp ( bt.bg )
            layer:insertProp ( bt.text )
        end
    else
        if self._infoString and self._infoString ~= "" then
            self.titleTextBox:addLoc ( 0, 20 )
            self._titleShift = true
            layer:insertProp ( self.infoTextBox )
        end
    end
    if animated then
        self.popupsRoot:addLoc ( -RealViewSize.width, 0 )
        self.popupsRoot:moveLoc ( RealViewSize.width, 0, 0.5 ):setListener ( MOAIAction.EVENT_STOP, function () app.queue:postEvent ( self, AnimEvents.MESSAGE_SHOWN ) end )
    else
        app.queue:postEvent ( self, AnimEvents.MESSAGE_SHOWN )
    end
end

function MessageBox:hide ( animated )
    --assert ( self._visible )
    local realHide = function (self)
        local layer = self._layer
        if self._titleShift then
            self.titleTextBox:addLoc ( 0, -20 )
            for _, bt in ipairs ( self._buttons ) do
                layer:removeProp ( bt.shadow )
                layer:removeProp ( bt.bg )
                layer:removeProp ( bt.text )
            end
            layer:removeProp ( self.infoTextBox )
        end
        layer:removeProp ( self.titleTextBox )
        layer:removeProp ( self.backGround )
        layer:removeProp ( self.backGroundShadow )
        self._visible = false
        self._infoString = ""
        self._hideAction = nil
    end
    if animated then
        local x0, y0 = self.popupsRoot:getLoc ()
        self._hideAction = self.popupsRoot:moveLoc ( RealViewSize.width, 0, 0.5 )
        self._hideAction:setListener ( MOAIAction.EVENT_STOP,
            function ()
                self.popupsRoot:setLoc ( x0, y0 )
                realHide ( self )
            end
        )
    else
        realHide ( self )
    end
end

function MessageBox:isVisible ()
    return self._visible
end

return MessageBox
