local core = require ( "core" )
local Resources = require ( "Resources" )
local SoundManager = require ( "SoundManager" )

local alphaBlendMode = { MOAIProp.GL_SRC_ALPHA, MOAIProp.GL_ONE_MINUS_SRC_ALPHA }
local defaulFont = "Resources/Fonts/trade_gothic.fnt"

local function onTouchEnter ( self )
    self:setScl ( 1.1 )
end

local function onTouchLeave ( self )
    self:setScl ( 1.0 )
end

local function onClick ( self )
    SoundManager.play ( "click.ogg" )
    if self.tapEvent then
        app.queue:postEvent ( self, self.tapEvent )
    end
end

local resources = 
{
    rm:rsrc ( Loaders.TileDeck, "buttonDeck" )
    {
        texture = "Resources/Images/darts_message_button.png",
        rect = { -58.5, -23.25, 58.5, 23.25 },
        size = { 1, 2 }
    };
    
    rm:rsrc ( Loaders.StretchPatch, "messageBoxBGDeck" )
    {
        texture = "Resources/Images/darts_message_bg.png",
        rows = {{ 0.25, false }, { 0.5, true }, { 0.25, false }},
        columns = {{ 0.25, false }, { 0.5, true }, { 0.25, false }},
        UVRects = {{ 0, 0.5, 1, 0 }, { 0, 1.0, 1, 0.5 }},
        rect = { -137, -61, 137, 61 },
    };
    
    rm:rsrc ( Loaders.Transform, "popupsRoot" )
    {
        rm:rsrc ( Loaders.Prop, "backGround" )
        {
            deck = "messageBoxBGDeck";
            rm:rsrc ( Loaders.Prop, "backGroundShadow" )
            {
                deck = "messageBoxBGDeck",
                color = { 0, 0, 0, 0.5 },
                blendMode = alphaBlendMode,
                loc = { -2, -3 },
            };
        };
        
        rm:rsrc ( Loaders.Transform, "buttonContainer" )
        {
            loc = { 0, -20 },
            rm:rsrc ( Loaders.Prop, "button_1" )
            {
                loc = { -58.5, 0 },
                deck = "buttonDeck",
                rm:rsrc ( Loaders.TextBox, "butttonT_1" )
                {
                    rect = { -58.5, -23.25, 58.5, 23.25 },
                    alignment = { MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY },
                    font = defaulFont,
                    textSize = 20,
                    yFlip = true,
                };
                rm:rsrc ( Loaders.Prop, "buttonShadow_1" )
                {
                    deck = "buttonDeck",
                    color = { 0, 0, 0, 0.5 },
                    blendMode = alphaBlendMode,
                    loc = { -2, -3 },
                };

                isClickable = true;
                onTouchEnter = onTouchEnter;
                onTouchLeave = onTouchLeave;
                onClick = onClick;
            };
            rm:rsrc ( Loaders.Prop, "button_2" )
            {
                loc = { 58.5 , 0 },
                deck = "buttonDeck",
                rm:rsrc ( Loaders.TextBox, "butttonT_2" )
                {
                    rect = { -58.5, -23.25, 58.5, 23.25 },
                    alignment = { MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY },
                    font = defaulFont,
                    textSize = 20,
                    yFlip = true,
                    clickSound = "click.ogg",
                };
                rm:rsrc ( Loaders.Prop, "buttonShadow_2" )
                {
                    deck = "buttonDeck",
                    color = { 0, 0, 0, 0.5 },
                    blendMode = alphaBlendMode,
                    loc = { -2, -3 },
                };

                isClickable = true;
                onTouchEnter = onTouchEnter;
                onTouchLeave = onTouchLeave;
                onClick = onClick;
            };
        };
        rm:rsrc ( Loaders.TextBox, "infoTextBox" )
        {
            loc = { 0, -20 },
            rect = { -137, -30, 137, 30 },
            alignment = { MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY },
            font = defaulFont,
            textSize = 20,
            textColor = { 1, 1, 1, 1 },
            yFlip = true,
        };
        rm:rsrc ( Loaders.TextBox, "titleTextBox" )
        {
            rect = { -137, -30, 137, 30 },
            alignment = { MOAITextBox.CENTER_JUSTIFY, MOAITextBox.CENTER_JUSTIFY },
            font = defaulFont,
            textSize = 35,
            textColor = { 1, 1, 1, 1 },
            yFlip = true,
        };
        loc = { RealViewSize.width * 0.5, RealViewSize.height * 0.6 },
    };
}

return resources
