local core = require ( "core" )

local BoardModel = require ( "Model.BoardModel" )

local ScoreView = core.class ( Object )
{
    PREFIXES = {
        [ BoardModel.R_BULLS_EYE ] = "",
        [ BoardModel.R_BULL ] = "",
        [ BoardModel.R_INNER_RING ] = "",
        [ BoardModel.R_TRIPLE_RING ] = "T",
        [ BoardModel.R_OUTER_RING ] = "",
        [ BoardModel.R_DOUBLE_RING ] = "D",
        [ BoardModel.R_FULL ] = "",
        [ BoardModel.R_NONE ] = "",
    }
}

local Events = require ( "Events" )
local PlayerEvents = Events.E_PLAYER
local AnimEvents = Events.E_ANIMATION
local ModelEvents = Events.E_MODEL

function ScoreView:addScoreBox ( textBox )
    local scoreBoxes = self._scoreBoxes
    if not scoreBoxes then
        scoreBoxes = {}
        self._scoreBoxes = scoreBoxes
    end
    scoreBoxes [ #scoreBoxes + 1 ] = textBox
end

function ScoreView:clearScoreBoxes ()
    self._scoreBoxes = nil
end

function ScoreView:clearScores ()
    for i = 1, #self._scoreBoxes do
        self:_displayScore ( i, "" )
    end
    self._currentIndex = nil
end

function ScoreView:displayNextScore ( value, sValue, circle )
    local currIdx = self._currentIndex or 1
    self:_displayScore ( currIdx, value, sValue, prefix )
    self._currentIndex = currIdx + 1
end

function ScoreView:_displayScore ( idx, value, sValue, circle )
    local str = value and tostring ( value ) or ""
    self._scoreBoxes [ idx ]:setString ( str )
end

function ScoreView:_displayScoreWithPrefix ( idx, value, sValue, circle )
    local prefix = circle and ScoreView.PREFIXES [ circle ]
    local str = ( prefix or "" ) .. ( sValue and tostring ( sValue ) or "" )
    self._scoreBoxes [ idx ]:setString ( str )
end

return ScoreView
