E_NET = Event.declare "net"
    :types
    {
        SESSION_STARTED = 0,

        READY = 1,
        START = 2,

		SPIN = 3,
		PERFORM = 4,
		RESULT = 5,
		ROUND = 6,
		
		VIDEO_STATE_CHANGED = 7,
		
		PAUSE = 8,
		RESUME = 9,
    }


AppCtrl = core.class ( Object )

function AppCtrl:ctor ()
    local fsm = FSM.new ( self )
    self.fsm = fsm

    self.model = Model.new ()
    
    app.queue:subscribe ( fsm, FSM.handleEvent, E_SYSTEM )
    app.queue:subscribe ( fsm, FSM.handleEvent, gui.E_ACTION )
    app.queue:subscribe ( fsm, FSM.handleEvent, gui.E_SCENE )
    app.queue:subscribe ( fsm, FSM.handleEvent, E_TIMEOUT )
    app.queue:subscribe ( fsm, FSM.handleEvent, E_NET )
    app.queue:subscribe ( fsm, FSM.handleEvent, nil, self.model )
	app.queue:subscribe ( fsm, FSM.handleEvent, WheelView.E_SPIN )
    app.queue:subscribe ( fsm, FSM.handleEvent, GameView.E_VIEW.PERFORM )
	app.queue:subscribe ( self, self.onVideoStateChanged, E_NET.VIDEO_STATE_CHANGED )	
	
	app.queue:subscribe ( self, self.onPause, E_SYSTEM.PAUSE )
	app.queue:subscribe ( self, self.onResume, E_SYSTEM.RESUME )
	app.queue:subscribe ( self, self.onFriendPause, E_NET.PAUSE )
	app.queue:subscribe ( self, self.onFriendResume, E_NET.RESUME )
	
    fsm:declare
    {
        {
            enter = function ( self )
            end;

            leave = function ( self )
                self.session = nil
                self.isMultiplayer = false
            end;

            [ E_SYSTEM.START ] = function ( self )
                return "INTRO"
            end;
        };

        INTRO =
        {
            enter = function ( self )
				-- self.view = GameView.new ( self.model, self.vsc )
				-- self.view:show ()
			
                self.splash = ScrSplash.new ()
                self.splash:show ()
				
				-- if tango then
					-- self.call = tango.videophone.Call_getActiveCall ()
					-- if self.call then
						-- self.vsc = self.call:getVideoStreamsControl ()
						-- if self.vsc then
							-- self.vsc:setSendingCamera ( tango.videophone.VideoStreamsControl_CT_FRONT )
							-- self.vsc:startSending ()
						-- end
					-- end
				-- end			
			
                -- self.view = GameView.new ( self.model, self.vsc )
				self.view = GameView.new ( self.model )
				
                self.channel = NetChannel.new ()
                if self.channel:open () then
                    -- set up observers
                    self.channel.opened:connect ( self,
                        function ( self, peerId )
                            app.queue:postEvent ( self, E_NET.SESSION_STARTED,
                                {
                                    id = MOAIEnvironment.generateGUID (),
                                    peerId = peerId,
                                }
                            )
                        end
                    )
                    self.channel.msgReceived:connect ( self,
                        function ( self, msgType, msgArgs )
                            app.queue:postEvent ( self, E_NET [ msgType ], msgArgs )
                        end
                    )
                end
				self._connectTimer = Timer.new ( 4.0 ):start ( 1 )
                self._waitingPlayersCnt = 2
				-- return "INIT"
            end;

            leave = function ( self )
                if self._connectTimer then
                    self._connectTimer:stop ()
                    self._connectTimer = nil
                end
                
                if self._waitingTimer then
                    self._waitingTimer:stop ()
                    self._waitingTimer = nil
                end
                
                self._waitingPlayersCnt = nil

                if not self.seed then
                    self.seed = self.model:generateSeed ()
                end
                self.model.isMultiplayer = self.isMultiplayer
                self.model:setup ( self.seed )
                self.seed = nil
            end;

            [ E_NET.SESSION_STARTED ] = function ( self, sender, event, args )
                self.session = args

                -- show "waiting message"

            end;

            [ E_NET.READY ] = function ( self, sender, event, args )
                local thisId = self.session.id
                local thatId = args.id
                local isHost = thisId < thatId
                
                self.session.isHost = isHost
                
                if isHost then
                    self.seed = self.model:generateSeed ()
                    print ( "I'm host.", self.seed )
                end

                self._waitingPlayersCnt = self._waitingPlayersCnt - 1
                if self._waitingPlayersCnt == 0 then
                    self.channel:send ( "START", { seed = isHost and self.seed or nil; } )
                end
            end;

            [ E_NET.START ] = function ( self, sender, event, args )
                -- confirmation received
                self.isMultiplayer = true
                if not self.session.isHost then
                    self.seed = args.seed
                    print ( "I'm not host.", self.seed )
                end

                return "INIT"
            end;

            [ E_TIMEOUT ] = function ( self, sender )
                if sender == self._connectTimer then
                    self._connectTimer = nil

                    if not self.session then
                        log ( "no session" )
                        return "INIT"
                    end

                    self.channel:send ( "READY", { id = self.session.id })
                    self._waitingTimer = Timer.new ( 10.0, sendReady )

                    self._waitingPlayersCnt = self._waitingPlayersCnt - 1
                    if self._waitingPlayersCnt == 0 then
                        self.channel:send ( "START", { seed = self.session.isHost and self.seed or nil; })
                    end
                end

                if sender == self._waitingTimer then
                    self.isMultiplayer = false
                    return "INIT"
                end
            end;
        };

        INIT = 
        {
            enter = function ( self )
                print "INIT"			
				
                self.model.isMultiplayer = self.isMultiplayer
                self.players = 
                {
                    [ Parties.PLAYER_1 ] = LocalPlayer.new  ( self.model, self.view, Parties.PLAYER_1 ),
                }
				
                self.splash:hide ()
				-- self.view:show ()
            end;

            [ gui.E_SCENE.SHOW ] = function ( self, sender )
                if sender == self.view then
                    return "SETUP"
                end
            end;

            [ gui.E_SCENE.HIDE ] = function ( self, sender )
                if sender == self.splash then
                    self.view:show ()
                end
            end;
        };
        
        SETUP =
            {
                enter = function ( self )
                    print "SETUP"
					
					local videoStatus = CameraCtrl.videoSupportStatus ()
					if videoStatus > CameraCtrl.DEVICE_TOO_OLD then
						self.vsc = CameraCtrl.getVsc ()
					end
					
					-- self:updateVideoPreview  ( false )
					-- self:updateVideoPlayback ( false )
					
					self.view:updateVideoPlaceholders ( videoStatus )

					self.showCameraDlg = false
					
					if not self.isMultiplayer and videoStatus >= CameraCtrl.VIDEO_SUPPORTED then
						self.view:updateVideoPlaceholders ( CameraCtrl.NOT_ON_CALL )
					else
						if videoStatus == CameraCtrl.VIDEO_SUPPORTED then
							self.showCameraDlg = true
						elseif videoStatus == CameraCtrl.VIDEO_LIVE then
							self:enableVideoPreview ()
						end
					end						
					
					self.currDlg = DlgHelp.new ( true )
					self.currDlg:show ()
					
                end;

				[ gui.E_ACTION ] = function ( self, sender, event, args )
					if args.id == "DlgExit.Exit" then
						av:playSound ( "art/sounds/click.ogg" )
						
						app.queue:postEvent ( nil, E_SYSTEM.QUIT, {})
						return "FINAL"
					end

					if args.id == "DlgHelp.Done" then
						av:playSound ( "art/sounds/click.ogg" )
						
						self.currDlg:hide ()
					end					

					if args.id == "DlgCamera.Yes" then
						av:playSound ( "art/sounds/click.ogg" )
						
						self.cameraDlg:hide ()
						self:enableVideoPreview ()
					end

					if args.id == "DlgCamera.No" then
						av:playSound ( "art/sounds/click.ogg" )

						self.cameraDlg:hide()
					end
				end;				
				
				[ gui.E_SCENE.HIDE ] = function ( self, sender )
					if sender == self.currDlg then
						self.currDlg = nil

						if self.showCameraDlg then
							self.cameraDlg = DlgCamera.new ( )
							self.cameraDlg:show ()				
						else
							return "GAME"
						end
					end
					
					if sender == self.cameraDlg then
						self.cameraDlg = nil
						return "GAME"
					end
				end;
				
				[ E_NET.ROUND ] = function ( self, sender, event, args )
					self.isFriendReady = true
				end;				

                leave = function ( self )
                end;
            };		
		
        GAME = 
        {
            enter = function ( self )
                -- self.model:startRound ()
				local model = self.model
				local player = self.players [ Parties.PLAYER_1 ]
				
				local isActive = not self.isMultiplayer or self.isMultiplayer and self.session.isHost
				
                self.players [ Parties.PLAYER_1 ]:setActive ( isActive )
				
				self.seed = model:generateSeed ()
				self.channel:send ( "ROUND", { seed = self.seed } )
				
				if not self.isMultiplayer or self.seed and self.isFriendReady then
					model:setup ( self.seed )
					model:startRound ()
					self.seed = nil
					self.isFriendReady = false
					self.view:showWaitingFriend ( not player.isActive )
				else
					self.view:showWaitingFriend ( true )
				end					
            end;
            
            leave = function ( self )
                self.view:showWaitingFriend ( false )
            end;

            [ Model.E_STATE_CHANGED ] = function ( self, sender, event, args )
                local model = sender
				local view = self.view
				local player = self.players [ Parties.PLAYER_1 ]

				if args.nextState == model.STATE_RESULT then
					local result = model:getResult ()
					if player.isActive then
						self.channel:send ( "RESULT", { result = result } )
					end
				end
				
                if args.nextState == model.STATE_ROUND_END then
				
					self.view:reset ( player.isActive )	
				
					-- local function onReset ()
						self.seed = model:generateSeed ()
						self.channel:send ( "ROUND", { seed = self.seed } )
						
						if not self.isMultiplayer or self.seed and self.isFriendReady then
							player:setActive ( not player.isActive )
							model:setup ( self.seed )
							model:startRound ()
							self.seed = nil
							self.isFriendReady = false
							self.view:showWaitingFriend ( not player.isActive )
						else
							self.view:showWaitingFriend ( true )
						end					
					-- end	
                end
            end;
			
			[ GameView.E_VIEW.PERFORM ] = function ( self, sender, event, args )
                self.channel:send ( "PERFORM", {} )
            end;
			
            [ gui.E_ACTION ] = function ( self, sender, event, args )
                if args.id == "DlgExit.Exit" then
                    app.queue:postEvent ( nil, E_SYSTEM.QUIT, {})
                    return "FINAL"
                end
            end;
			
			[ WheelView.E_SPIN.START ] = function ( self, sender, event, args )
				self.channel:send ( "SPIN", { speed = args.speed, rotation = args.rotation } )
			end;			
			
            [ E_NET.SPIN ] = function ( self, sender, event, args )
				self.players [ Parties.PLAYER_1 ]:spin ( args.speed, args.rotation )
            end;
			
            [ E_NET.PERFORM ] = function ( self, sender, event, args )
				self.players [ Parties.PLAYER_1 ]:showRate ()
            end;
			
            [ E_NET.RESULT ] = function ( self, sender, event, args )
				self.players [ Parties.PLAYER_1 ]:setResult ( args.result )
            end;
			
            [ E_NET.ROUND ] = function ( self, sender, event, args )
                local model = self.model
				local player = self.players [ Parties.PLAYER_1 ]
				
                if args.seed then
                    self.seed = args.seed
                end
				
				self.isFriendReady = true
				
                if model.state ~= model.STATE_ROUND_END and model.state ~= model.STATE_READY then
                    return
                end
				
				if not self.isMultiplayer or self.seed and self.isFriendReady then
					if model.state == model.STATE_ROUND_END then
						player:setActive ( not player.isActive )
					end
					model:setup ( self.seed )
					model:startRound ()
					self.seed = nil
					self.isFriendReady = false
					self.view:showWaitingFriend ( not player.isActive )
				else
					self.view:showWaitingFriend ( true )
				end
            end;

            [ E_SYSTEM.QUIT ] = function ( self )
                return "FINAL"
            end;
        };
        
        
        GAME_OVER = 
        {
            enter = function ( self )
                --return "FINAL"
                -- local dlg = GameOverDialog.new ( self.model )
                -- dlg:show ()
            end;

            [ gui.E_ACTION ] = function ( self, sender, event, args )
                -- if args.id == "GameOverDialog.Exit" then
                    -- return "FINAL"
                -- end
            end;
        };

        FINAL = 
        {
            enter = function ( self )
                local exit = MOAISim.exit or os.exit
                exit ()
            end;
        };
    }
end

function AppCtrl:onPause ()
	if self.channel then
		self.channel:send ( "PAUSE", {} )
	end
end

function AppCtrl:onResume ()
	if self.channel then
		self.channel:send ( "RESUME", {} )
	end
end

function AppCtrl:onFriendPause ()
	self.hadVideoPlayback = self.view.hasVideoPlayback
	self:updateVideoPlayback ( false )
end

function AppCtrl:onFriendResume ()
	self:updateVideoPlayback ( self.hadVideoPlayback )
end

function AppCtrl:enableVideoPreview ()
	CameraCtrl.enableOutgoingVideo ()
	self:updateVideoPreview ( true )
	self.channel:send ( "VIDEO_STATE_CHANGED", { enabled = true } )
end

function AppCtrl:updateVideoPreview ( hasVideo )
    -- hasVideo = hasVideo or ( self.vsc and self.vsc:isSending() )
    self.view:updateVideoPreview ( hasVideo )
end

function AppCtrl:updateVideoPlayback ( hasVideo )
    -- hasVideo = hasVideo or ( self.vsc and self.vsc:isReceiving() )
    self.view:updateVideoPlayback ( hasVideo )
end

function AppCtrl:onVideoStateChanged ()
	local videoStatus = CameraCtrl.videoSupportStatus ()
	local isVideoSupported = videoStatus >= CameraCtrl.VIDEO_SUPPORTED
	self:updateVideoPlayback ( isVideoSupported )
end