Parties =
{
    PLAYER_1 = 1,
    PLAYER_2 = -1,
}

Model = core.class ( Object )
{
    E_STATE_CHANGED = Event.declare "Model.StateChanged";

    STATE_INIT              = 0,
    STATE_READY             = 1,

    STATE_ROUND_BEG         = 3,
    STATE_SPINNING          = 4,
    STATE_ACTION            = 5,
	STATE_RESULT			= 6,
    STATE_ROUND_END         = 7;
}

function Model:ctor ()
    self.state = self.STATE_INIT

    self._score =
    {
        [ Parties.PLAYER_1 ] = 0,
        [ Parties.PLAYER_2 ] = 0,
    }
end

function Model:_setState ( nextState )
    local prevState = self.state
    self.state = nextState
    log ( "Model.state %d -> %d", prevState, nextState )
    app.queue:postEvent ( self, self.E_STATE_CHANGED, { prevState = prevState, nextState = nextState })
end

function Model:generateSeed ()
    return tonumber(tostring(os.time()):reverse():sub(1, 6))
end

function Model:setup ( seed )
    assert ( self.state == self.STATE_INIT or self.state == self.STATE_ROUND_END or self.state == self.STATE_READY )
    
	math.randomseed ( seed )
	
    self:_setState ( self.STATE_READY )
    return seed
end

function Model:startRound ()
    assert ( self.state == self.STATE_READY )
    -- self.round = self.round + 1
	
	self._result = nil
	
    self:_setState ( self.STATE_ROUND_BEG )
end

function Model:startSpinning ()
    assert ( self.state == self.STATE_ROUND_BEG )
    self:_setState ( self.STATE_SPINNING )
end

function Model:chooseAction ( action )
	assert ( self.state == self.STATE_SPINNING )
	
	self._action = action
	
	self:_setState ( self.STATE_ACTION )
end

function Model:getAction ()
	return self._action
end

function Model:setResult ( player, result )
	if self._result ~= nil then
		return
	end
	
	self._result = result

	if result then
		self._score [ player ] = self._score [ player ] + 1
	end
	
	log ( "_score = [%d, %d] ", self._score [ Parties.PLAYER_1 ], self._score [ Parties.PLAYER_2 ] )
	
	self:_setState ( self.STATE_RESULT )
end

function Model:getResult ()
	return self._result
end

function Model:getScore ()
	return self._score [ Parties.PLAYER_1 ], self._score [ Parties.PLAYER_2 ]
end

function Model:endRound ()
    self:_setState ( self.STATE_ROUND_END )
end


