GameOverDialog = core.class ( gui.Scene )

local TRANSITION_DURATION = 0.5

local RSRC_POPUP_DECOR_WDG = rm:rsrc ( gui.Widget )
{
    rotation = 90;
    color = { 1, 1, 1, 0.2 },

    deck = rm:rsrc ( gui.GfxQuadListDeck )
    {
        [ 1 ] =
        {
            { sprite = "central_decor.png"; y = 220; scaleX = 1, scaleY =-1; },
            { sprite = "central_decor.png"; y =-220; scaleX = 1, scaleY = 1; }
        }
    };
}


local RSRC_OVERLAY_WDG = rm:rsrc ( gui.Widget, "/wdgOverlay" )
{
    color = { 0, 0, 0, 0 };
    deck = rm:rsrc ( gui.GfxQuad )
    {
        sprite = "white_16x16.png";
        scaleX = 100.0, scaleY = 100.0;
    };
}

local BG_DECK = rm:rsrc ( gui.GfxQuad )
{
    sprite = "popup.png";
    scaleX = 2.0, scaleY = 2.0;
}

local RSRC_TITLE_WDG = rm:rsrc ( gui.TextBox, "/txtTitle" )
{
    x = 0, y = 70;
    color = { 0.9296875, 0.8515625, 0.38671875 };
    text = "GAME OVER";
    font = "art/fonts/palatino.fnt",
    fontSize = 68;
    rect = { -300, -85, 300, 85 };
}

local RSRC_SP_VIEW = rm:rsrc ( gui.Widget )
{
    x = 0, y = 0;
    rotation = 90;

    RSRC_OVERLAY_WDG,

    rm:rsrc ( gui.Widget, "/wdgBG" )
    {
        deck = BG_DECK,
        RSRC_TITLE_WDG,
        RSRC_POPUP_DECOR_WDG,

        rm:rsrc ( gui.TextBox, "/txtScore1" )
        {
            x = 0, y = -20;
            color = { 0.9296875, 0.8515625, 0.38671875 };
            font = "art/fonts/palatino.fnt",
            fontSize = 50;
            rect = { -200, -85, 200, 85 };
        },

        rm:rsrc ( gui.TextBox, "/txtScore2" )
        {
            x = 0, y = -70;
            color = { 0.9296875, 0.8515625, 0.38671875 };
            font = "art/fonts/palatino.fnt",
            fontSize = 50;
            rect = { -200, -85, 200, 85 };
        },

        rm:rsrc ( gui.TextBox, "/txtHint" )
        {
            x = 0, y = -200;
            font = "art/fonts/palatino.fnt",
            fontSize = 40;
            rect = { -200, -85, 200, 85 };
        }
    },
}

local RSRC_MP_VIEW = rm:rsrc ( gui.Widget )
{
    x = 0, y = 0;
    rotation = 90;

    RSRC_OVERLAY_WDG,

    rm:rsrc ( gui.Widget, "/wdgBG" )
    {
        deck = BG_DECK,
        RSRC_TITLE_WDG,
        RSRC_POPUP_DECOR_WDG,

        rm:rsrc ( gui.TextBox, "/txtScore1" )
        {
            x = -10, y = -20;
            text = "You:";
            color = { 0.9296875, 0.8515625, 0.38671875 };
            font = "art/fonts/palatino.fnt",
            fontSize = 50;
            textAlignX = MOAITextBox.RIGHT_JUSTIFY;
            rect = { -200, -85, 0, 85 };
        },

        rm:rsrc ( gui.TextBox, "/txtScore2" )
        {
            x = -10, y = -70;
            text = "Friend:";
            color = { 0.9296875, 0.8515625, 0.38671875 };
            font = "art/fonts/palatino.fnt",
            fontSize = 50;
            textAlignX = MOAITextBox.RIGHT_JUSTIFY;
            rect = { -200, -85, 0, 85 };
        },

        rm:rsrc ( gui.TextBox, "/txtScore1" )
        {
            x = 10, y = -20;
            font = "art/fonts/palatino.fnt",
            fontSize = 50;
            textAlignX = MOAITextBox.LEFT_JUSTIFY;
            rect = { 0, -85, 200, 85 };
        },

        rm:rsrc ( gui.TextBox, "/txtScore2" )
        {
            x = 10, y = -70;
            font = "art/fonts/palatino.fnt",
            fontSize = 50;
            textAlignX = MOAITextBox.LEFT_JUSTIFY;
            rect = { 0, -85, 200, 85 };
        },

        rm:rsrc ( gui.TextBox, "/txtHint" )
        {
            x = 0, y = -200;
            font = "art/fonts/palatino.fnt",
            fontSize = 40;
            rect = { -200, -85, 200, 85 };
        }
    },
}

function GameOverDialog:ctor ( model )
    local root = model.isMultiplayer and RSRC_MP_VIEW:create () or RSRC_SP_VIEW:create ()

    gui.Scene.ctor ( self, root )

    if model.isMultiplayer then
        local score1, score2 = model:getBalance ( Parties.PLAYER_1 ), model:getBalance ( Parties.PLAYER_2 )
        root.txtScore1.prop:setString ( string.format ( "$ %d", score1 ))
        root.txtScore2.prop:setString ( string.format ( "$ %d", score2 ))
    else
        root.txtScore1.prop:setString ( "You have lost" )
        root.txtScore2.prop:setString ( "all your money" )
    end
end

function GameOverDialog:onShowTransition ()
    local root = self.root
    local dx, dy = 0, 600
    root.wdgBG.transform:moveLoc ( dx, dy )
    root.wdgBG.transform:moveLoc ( dx, -dy, TRANSITION_DURATION )
        :setListener ( MOAIAction.EVENT_STOP, 
            function ()
                self:onShow ()
            end
        )

    root.wdgOverlay.prop:seekColor ( 0, 0, 0, 0.7, TRANSITION_DURATION )

    Timer.new ( 2.5,
        function ()
            root.txtHint.prop:setString ( "Tap to continue" )
        end
    ):start ( 1 )
end

function GameOverDialog:onHideTransition ()
    local root = self.root
    local dx, dy = 0, 600
    root.wdgBG.transform:moveLoc ( dx, dy, TRANSITION_DURATION )
        :setListener ( MOAIAction.EVENT_STOP, 
            function ()
                self:onHide ()
            end
        )

    root.wdgOverlay.prop:seekColor ( 0, 0, 0, 0.0, TRANSITION_DURATION )
end

function GameOverDialog:onTouchBegan ( sender, event, args )
    local touches = args.touches
    if touches.n > 1 then
        return
    end

    app.queue:postEvent ( self, gui.E_ACTION, { id = "GameOverDialog.Exit"})
end

