local function makeBG ( fill )
	local rsrc = rm:rsrc ( gui.Widget, "/background" ) 
	{
		x = -0.5 * MODEL_SCREEN_W, y = -0.5 * MODEL_SCREEN_H;
		
		deck = rm:rsrc ( gui.TileDeck ) 
		{ 
			texture = "art/textures/bg.png";
			rows = 1, cols = 1;
		},
		
		grid = rm:rsrc ( gui.RectGrid ) 
		{ 
			tileWidth = 37.0, tileHeight = 37.0;
			width = math.ceil ( MODEL_SCREEN_W / 37.0 );
			height = math.ceil ( MODEL_SCREEN_H / 37.0 );
			fill = fill;
		}
	}
	
    return rsrc
end

local function animateWheel ( target, rotation, period )
    local curve = MOAIAnimCurve.new ()
    curve:reserveKeys ( 2 )
    curve:setKey ( 1, 0.0, 0.0, MOAIEaseType.LINEAR )
    curve:setKey ( 2, period, rotation )
    curve:setWrapMode ( MOAIAnimCurve.WRAP )

    local anim = MOAIAnim.new ()
    anim:reserveLinks ( 1 )
    anim:setLink ( 1, curve, target.prop, MOAITransform.ATTR_Z_ROT )
    anim:setMode ( MOAITimer.CONTINUE )
    return anim
end

local RSRC_ROOT = rm:rsrc ( gui.Widget )
{
    makeBG ( 1 ),

	rm:rsrc ( gui.Widget, "/wheels" )
	{
		rm:rsrc ( gui.Widget )
		{
			x = - 5.0;
			y = 0.5 * MODEL_SCREEN_H - 130.0;
			color = { 1.0, 1.0, 1.0, 0.175 };
			blend = { src = MOAIProp2D.GL_SRC_ALPHA, dst = MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA };
		
			deck = rm:rsrc ( gui.GfxQuad )
			{
				sprite = "tango_logo.png";
				scaleX = 0.375, scaleY = 0.375;
			};
		
			rm:rsrc ( gui.Widget, "/wheel1" )
			{
				piv = { 0.25, 0.0 };
				
				blend = { src = MOAIProp2D.GL_SRC_ALPHA, dst = MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA };
				deck = rm:rsrc ( gui.GfxQuad )
				{
					sprite = "wheel_spinning.png";
					scaleX = 0.75, scaleY = 0.75;
				};
			},
		},

		rm:rsrc ( gui.Widget )
		{	
			x = - 0.5 * MODEL_SCREEN_W + 50.0;
			y = 0.5 * MODEL_SCREEN_H - 15.0;
			color = { 1.0, 1.0, 1.0, 0.15 };
			blend = { src = MOAIProp2D.GL_SRC_ALPHA, dst = MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA };				
				
			deck = rm:rsrc ( gui.GfxQuad )
			{
				sprite = "tango_logo.png";
				scaleX = 0.925, scaleY = 0.925;
			};
			
			rm:rsrc ( gui.Widget, "/wheel2" )
			{
				piv = { 0.45, -1.125 };
			
				blend = { src = MOAIProp2D.GL_SRC_ALPHA, dst = MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA };
							
				deck = rm:rsrc ( gui.GfxQuad )
				{
					sprite = "wheel_spinning.png";
					scaleX = 1.85, scaleY = 1.85;
				};
			},
		},
		
		rm:rsrc ( gui.Widget )
		{
			x = 0.5 * MODEL_SCREEN_W - 120.0;
			y = - 0.5 * MODEL_SCREEN_H + 65.0;
			color = { 1.0, 1.0, 1.0, 0.15 };
			blend = { src = MOAIProp2D.GL_SRC_ALPHA, dst = MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA };
			
			deck = rm:rsrc ( gui.GfxQuad )
			{
				sprite = "tango_logo.png";
				scaleX = 1.2, scaleY = 1.2;
			};
			
			rm:rsrc ( gui.Widget, "/wheel3" )
			{
				piv = { 0.6, -1.5 };
				
				blend = { src = MOAIProp2D.GL_SRC_ALPHA, dst = MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA };
				
				deck = rm:rsrc ( gui.GfxQuad )
				{
					sprite = "wheel_spinning.png";
					scaleX = 2.4, scaleY = 2.4;
				};
			},
		},

		rm:rsrc ( gui.Widget )
		{
			x = - 0.5 * MODEL_SCREEN_W + 160;
			y = - 0.5 * MODEL_SCREEN_H + 150;
			color = { 1.0, 1.0, 1.0, 0.175 };
			blend = { src = MOAIProp2D.GL_SRC_ALPHA, dst = MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA };
				
			deck = rm:rsrc ( gui.GfxQuad )
			{
				sprite = "tango_logo.png";
				scaleX = 0.325, scaleY = 0.325;
			};
				
			rm:rsrc ( gui.Widget, "/wheel4" )
			{
				piv = { 0.25, 0.0 };
			
				blend = { src = MOAIProp2D.GL_SRC_ALPHA, dst = MOAIProp2D.GL_ONE_MINUS_SRC_ALPHA };
			
				deck = rm:rsrc ( gui.GfxQuad )
				{
					sprite = "wheel_spinning.png";
					scaleX = 0.65, scaleY = 0.65;
				};				
			},			
		},	
	},
	
	rm:rsrc ( gui.Widget, "/logo" )
	{
		x = 0, y = -10;

		deck = rm:rsrc ( gui.GfxQuad )
		{
			sprite = "splash_logo.png"; 
			scaleX = 2, scaleY = 2;
		};
	},

	rm:rsrc ( gui.Widget, "/moai_logo" )
	{
		x = 0.5 * MODEL_SCREEN_W - 165;
		y = - 0.5 * MODEL_SCREEN_H + 75;

		deck = rm:rsrc ( gui.GfxQuad )
		{
			sprite = "moai_logo.png"; 
			scaleX = 1, scaleY = 1;
		};
	},
}


ScrSplash = core.class ( gui.Scene )

function ScrSplash:ctor ()
    local root = RSRC_ROOT:create ()
    gui.Scene.ctor ( self, root )
end

function ScrSplash:onShowTransition ()
    local root = self.root

	self.wheelAnimations = 
	{
		animateWheel ( root.wheel1, 360.0, 2.0 ) : start (),
		animateWheel ( root.wheel2, -360.0, 3.0 ) : start (),
		animateWheel ( root.wheel3, -360.0, 4.0 ) : start (),
		animateWheel ( root.wheel4, 360.0, 1.0 ) : start (),
	}
        
    self:onShow ()
end

function ScrSplash:onHideTransition ()
    local root = self.root

    -- root.logo.prop:seekColor ( 0, 0, 0, 0, 1.0 )

    -- for w in root.wheels:iterChildren () do
        -- w.prop:seekColor ( 0, 0, 0, 0, 1.0 )
    -- end
	
    -- Timer.new ( 1.25,
        -- function ()
            -- self:onHide ()

			-- for _, anim in ipairs ( self.wheelAnimations ) do
				-- anim:stop ()
			-- end
        -- end
    -- ):start ( 1 )
	
	self:onHide ()
	
end
