WheelView = core.class ( gui.Widget )
{
	SECTOR_COUNT = 8,
	
	MAX_TOUCH_QUEUE_LENGTH = 10,
	
	MIN_SPEED = 500.0,

	MAX_LOW_SPEED = 500.0,
	MAX_HIGH_SPEED = 1500.0,
	
	EPS = 300.0,
	
	SECTOR_COLOR =
	{
		{ r = 127, g =   0, b = 183 },
		{ r = 185, g =   0, b = 212 },
		{ r = 215, g =   0, b =  11 },
		{ r = 232, g =  83, b =   0 },
		
		-- { r = 249, g = 238, b =   0 },
		{ r = 255, g = 215, b =   0 },
		
		{ r = 102, g = 249, b =   0 },
		{ r =   0, g = 234, b = 230 },
		{ r =   0, g =  79, b = 255 },
	},
	
	REGULAR  = 1,
	SPINNING = 2,
	-- DISABLED = 3,
	
	E_SPIN = Event.declare "WheelView"
		:types
		{
			START = 1,
			STOP  = 2,
		};
}

function WheelView:ctor ( rsrc, context )
    gui.Widget.ctor ( self, rsrc, context )

    assert ( rsrc.id )
    self.id = rsrc.id
	
	self.center = { x = rsrc.x or 0, y = rsrc.y or 0} 
	
	local innerRadius = rsrc.innerRadius or 0
	local outerRadius = rsrc.outerRadius or 0
	
	self.innerRadius2 = innerRadius * innerRadius
	self.outerRadius2 = outerRadius * outerRadius
	
	self._state = self.REGULAR
	-- self._state = self.DISABLED
	self._isSpinning = false
	self._isEnabled = false
	
	self.started = core.signal ()
	self.rejected = core.signal ()
	self.stopped = core.signal ()
	
	local i = 0
	local r = 171
	for w in self.smiles:iterChildren () do
		local rot = i * 360.0 / self.SECTOR_COUNT
		w.prop:setColor ( 1.0, 1.0, 1.0, 0.6 )
		w.transform:setLoc ( r * math.sin ( math.rad ( rot ) ), r * math.cos ( math.rad ( rot ) ) )
		w.transform:setRot ( - rot )
		i = i + 1
    end
	
	self:updateLook ()
end

-- function WheelView:_update ()
    -- local state
    -- -- if self._isEnabled then
        -- -- state = self._isSpinning and self.SPINNING or self.REGULAR
    -- -- else
        -- -- state = self.DISABLED
    -- -- end

    -- if self._state == state then
        -- return
    -- end

    -- self._state = state
    -- self:updateLook ()
-- end

function WheelView:setActive ( isActive )
    -- if isActive then
		-- self.prop:setColor ( 1.0, 1.0, 1.0, 1.0 )
    -- else
        -- self.prop:setColor ( 0.5, 0.5, 0.5, 0.5 )
    -- end
end

function WheelView:setEnabled ( isEnabled )
    if not isEnabled == not self._isEnabled then
        return
    end
	
    if not isEnabled then
        self._touchIndex = nil
        self._isEnabled = nil
    end

    self._isEnabled = isEnabled
	self:updateLook ()
end

function WheelView:getRadius ( x, y )
	return x - self.center.x, y - self.center.y
end

function WheelView:getTouch ( x, y )
	local rx, ry = self:getRadius( x, y );
	local touch =
	{
		angle = math.deg ( math.atan2 ( ry, rx ) ),
		timestamp = MOAISim.getDeviceTime (),
	}
	return touch
end

function WheelView:acceptsTouch ( x, y )
	if self._state ~= self.REGULAR or not self._isEnabled then
		return false
	end

	if self._isSpinning then
		return false
	end
	
	local rx, ry = self:getRadius( x, y );
	local r2  = rx * rx + ry * ry

	return r2 > self.innerRadius2 and r2 < self.outerRadius2
end

function WheelView:onTouchBegan ( sender, event, args )
    local touches = args.touches
    if touches.n > 1 then
        return
    end
    self._touchIndex = args.index
		
	self._touchQueue = { self:getTouch ( args.x, args.y ) }
	
	-- log("onTouchBegan")
end

function WheelView:onTouchMoved ( sender, event, args )
    if args.index ~= self._touchIndex then
        return
    end
	
	table.insert ( self._touchQueue, 1, self:getTouch ( args.x, args.y ) )
	
	if #self._touchQueue > self.MAX_TOUCH_QUEUE_LENGTH then
		table.remove ( self._touchQueue )
	end
	
	if #self._touchQueue > 1 then
		local currTouch = self._touchQueue [ 1 ]
		local prevTouch = self._touchQueue [ 2 ]
		
		local rotation = currTouch.angle - prevTouch.angle
		self.transform:addRot ( rotation )
	end
end

function WheelView:onTouchEnded ( sender, event, args )
    if args.index ~= self._touchIndex then
        return
    end

	if #self._touchQueue > 1 then
		local firstTouch = self._touchQueue [ 1 ]
		local lastTouch = self._touchQueue [ #self._touchQueue ]
		
		local rotation = lastTouch.angle - firstTouch.angle
		
		-- log ( "rotation 1 = %f", rotation )
		
		if rotation > 180.0 then
			rotation = rotation - 360.0
		elseif rotation < -180.0 then
			rotation = rotation + 360.0
		end
		
		local timeSpan = math.abs ( lastTouch.timestamp - firstTouch.timestamp )
		local speed = rotation / timeSpan
		
		-- log ( "rotation 2 = %f", rotation )
		-- log ( "angle: %f => %f :: time: %f => %f", firstTouch.angle, lastTouch.angle, firstTouch.timestamp, lastTouch.timestamp)
		
		-- local action = self:getAction ( self.transform:getRot () )
		-- log ( "\n\nACTION = %d\n\n", action ) 
		
		if math.abs ( speed ) < self.MIN_SPEED then 
			self.rejected ()
			return 
		end
		
		local rotation = self.transform:getRot ()
		app.queue:postEvent ( self, WheelView.E_SPIN.START, { speed = speed, rotation = rotation } )
		
		self:spin ( speed )
	end
end

function WheelView:getAction ( rotation )
	local sectorSize = 360.0 / self.SECTOR_COUNT
	local sector = math.floor ( rotation / sectorSize + 0.5 )
	return sector % 8 + 1
end

function WheelView:playSound ( duration )

	av:playSound ( "art/sounds/wheel.ogg" )
	
	-- local timer = MOAITimer.new ()
	-- timer:setSpan ( duration )
	-- timer:start ()

	-- local function threadFunc ()
		-- while timer:isBusy() do
			-- local snd = av:getSound ( "art/sounds/spin.ogg" )
			-- if snd then
				-- snd:play ()
				-- while snd:isPlaying () do coroutine:yield () end			
			-- end
			
			-- local start = duration - 3.0
			-- local span = timer:getTime ()
			
			-- if span > start then
				-- local timer = MOAITimer.new ()
				-- timer:setSpan ( math.exp ( span - start ) / 5.0 )
				-- timer:start ()
				-- while timer:isBusy () do coroutine:yield () end
			-- end
		
			-- coroutine:yield () 
		-- end
	-- end

	-- thread = MOAIThread.new ()
	-- thread:run ( threadFunc )
	
	-- -- -- -- -- -- -- -- -- -- -- -- --
	
	-- log ( "duration = %f", duration )
	
	-- local duration = math.floor ( duration / 0.5 ) - 1
	
	-- duration = math.max ( duration, 1 )
	-- duration = math.min ( duration, 10 )
	
	-- -- log ( "start = %d", 11 - duration )
	
	
	-- local function wait ( snd )
		-- if not snd then
			-- return
		-- end
		-- while snd:isPlaying () do coroutine:yield () end
	-- end

	-- local function threadFunc ()
		-- for i = 11 - duration, 10 do
			-- local filename = string.format ( "art/sounds/wheel%d.ogg", i )
			-- -- log ( "Play: " .. filename )
			-- wait ( av:playSound ( filename ) )
		-- end
	-- end

	-- thread = MOAIThread.new ()
	-- thread:run ( threadFunc )	
end

function WheelView:spin ( speed, rotation )	
	self._isSpinning = true
	
	self:playSound ( math.abs ( speed ) / self.EPS )
	
	self.started ()
	
	if math.abs ( speed ) < self.MAX_LOW_SPEED then
		self:startSlowSpinning ( speed, rotation, true )
	else
		self:startFastSpinning ( speed, rotation )
	end
end

function WheelView:startSlowSpinning ( speed, startRot, start )
	if self._state == self.SPINNING then
		self._state = self.REGULAR
		self:updateLook ()
	end
	
	self.spinning.prop:seekColor ( 1.0, 1.0, 1.0, 0.0, 1.0, MOAIEaseType.EASE_IN )
	
	local direction = - math.sign ( speed )
	local duration = math.abs ( speed ) / self.EPS
	local currRot = self.transform:getRot ()
	local desiredRot = direction * self.EPS * duration * duration / 2.0 + currRot
	
	if startRot then
		local delta = ( startRot - currRot ) % 360.0
		if math.abs ( delta ) > 180.0 then
			delta = delta - math.sign ( delta ) * 360.0
		end
		desiredRot = desiredRot + delta
	end
	
	local sector = self:getAction ( desiredRot )
	local sectorSize = 360.0 / self.SECTOR_COUNT
	local rotation = ( sector - 1 ) * sectorSize + math.floor ( desiredRot / 360.0 ) * 360.0
	
	-- log ( "\n\nROTATION: %f => %f\n", rotation, desiredRot )
	-- log ( "\n\nSECTOR: %d => %d\n", sector, self:getAction ( rotation ) )

	-- log ( "\n\nrotation = %f\n\n", rotation )
	
	local maxSpan = math.abs ( self.MAX_HIGH_SPEED ) / self.EPS
	local span = start and maxSpan or duration
    local action = self.transform:seekRot ( rotation, span, MOAIEaseType.EASE_IN )
    action:setListener ( MOAIAction.EVENT_STOP, 
        function ()
			self._isSpinning = false
			
			self:setEnabled ( false )
			
			local rot = self.transform:getRot ()
			local action = self:getAction ( rot )
			
			self.stopped ( action )
			
			self:onStopSpinning ( action )
			-- log ( "end slow spinning :: rot = %f", rot )
        end
	)
	
	return action
end

function WheelView:startFastSpinning ( speed, startRot )
	if self._state == self.REGULAR then
		self._state = self.SPINNING
		self:updateLook ()
	end
	
	self.spinning.prop:setColor ( 1.0, 1.0, 1.0, 1.0 )
	
	local direction = - math.sign ( speed )
	local speed = math.min( math.abs( speed ), self.MAX_HIGH_SPEED ) 
	local duration = ( self.MAX_HIGH_SPEED - self.MAX_LOW_SPEED ) / self.EPS
	local currRot = self.transform:getRot ()
	local rotation = direction * duration * ( self.MAX_LOW_SPEED + self.EPS * duration / 2.0 )
	
	if startRot then
		rotation = rotation + ( startRot - currRot ) % 360.0
	end
	
	local function wait ( action )
		while action:isBusy () do coroutine:yield () end
	end

	local function threadFunc ()
		wait ( self.transform:moveRot ( rotation, duration, MOAIEaseType.LINEAR ) )
		wait ( self:startSlowSpinning ( - direction * self.MAX_LOW_SPEED ) )
	end

	thread = MOAIThread.new ()
	thread:run ( threadFunc )	
end

function WheelView:onStopSpinning ( action )
	-- if self.sound then
		-- self.sound:stop ()
	-- end

	local smiles = self.smiles:getChildren ()
	local smile = smiles [ action ]
	
	local sizeCurve = MOAIAnimCurve.new ()
    sizeCurve:reserveKeys ( 2 )
    sizeCurve:setKey ( 1, 0.0, 1.0 )
    sizeCurve:setKey ( 2, 1.0, 1.1 )
	
	local colorCurve = MOAIAnimCurve.new ()
    colorCurve:reserveKeys ( 2 )
    colorCurve:setKey ( 1, 0.0, 0.7 )
    colorCurve:setKey ( 2, 1.0, 1.0 )	
	
	local anim = MOAIAnim.new ()
	anim:reserveLinks ( 3 )
	anim:setLink ( 1, colorCurve, smile.prop, MOAIColor.ATTR_A_COL )	
	anim:setLink ( 2, sizeCurve, smile.prop, MOAITransform.ATTR_X_SCL )
	anim:setLink ( 3, sizeCurve, smile.prop, MOAITransform.ATTR_Y_SCL )
	
	local function onStop ( self )
		app.queue:postEvent ( self, WheelView.E_SPIN.STOP, { action = action } )
	end
	
	anim:setListener ( MOAIAction.EVENT_STOP, onStop )
	
	anim:start ()	
	
	-- smile.prop:seekColor ( 1.0, 1.0, 1.0, 1.0, 1.0, MOAIEaseType.EASE_OUT )
	-- smile.prop:seekScl ( 1.1, 1.1, 1.0, MOAIEaseType.EASE_OUT )

end

function WheelView:reset ()
	for w in self.smiles:iterChildren () do
		w.prop:setColor ( 1.0, 1.0, 1.0, 0.6 )
		w.prop:setScl ( 1.0, 1.0 )	
		-- w.prop:seekColor ( 1.0, 1.0, 1.0, 0.6, 0.5,  MOAIEaseType.EASE_OUT )
		-- w.prop:seekScl ( 1.0, 1.0, 0.5, MOAIEaseType.EASE_OUT )
    end
end

function WheelView:updateLook ()
    local state = self._state
	
	if not state or state == self.REGULAR then
		self.regular.prop:setVisible ( true )
		self.spinning.prop:setVisible ( false )
	elseif state == self.SPINNING then
		self.regular.prop:setVisible ( false )
		self.spinning.prop:setVisible ( true )
	end
end

