-- library
require "lib.core"
require "lib.gui"
require "lib.av"
-- model
require "src.model.LCG"
require "src.model.Model"

-- views+
require "src.view.MenuButton"
require "src.view.WordBoard"
require "src.view.Thimble"
require "src.view.GameTimer"
require "src.view.PlayerScore"
require "src.view.SplashScreen"
require "src.view.GameView"
require "src.view.ExitDialog"
require "src.view.InfoDialog"
require "src.view.EnableVideoDialog"

-- controllers
require "src.controller.PlayerCtrl"
require "src.controller.LocalPlayer"
require "src.controller.AIPlayer"
require "src.controller.NetPlayer"
require "src.controller.AppCtrl"
require "src.controller.CameraCtrl"


SCREEN_SIZE = { 640, 960 }

TOP_PANEL_OFFSET_Y              = 0
TOP_PANEL_OFFSET_X              = 0
RESIZE_SCALE_Y                  = 1
RESIZE_SCALE_X                  = 1

MODEL_SCREEN_W = 0
MODEL_SCREEN_H = 0


local function setupViewport ()
  --  local screenW, screenH = SCREEN_SIZE [ 1 ], SCREEN_SIZE [ 2 ]
    
    local screenWidth, screenHeight = MOAIGfxDevice.getViewSize ()
    
    -- screenWidth, screenHeight = 320, 480
    -- screenWidth, screenHeight = 480, 800
    -- screenWidth, screenHeight = 640, 960
    -- screenWidth, screenHeight = 640, 1136
    -- screenWidth, screenHeight = 320, 568
    -- screenWidth, screenHeight = 640, 1036
     -- screenWidth, screenHeight = 768, 1024
    -- screenWidth, screenHeight = 384, 512
    --screenWidth, screenHeight = 380, 800
    
    screenWidth, screenHeight = screenWidth or 0, screenHeight or 0
    if screenWidth == 0 or screenHeight == 0 then
        screenWidth, screenHeight = SCREEN_SIZE [ 1 ] * 0.5, SCREEN_SIZE [ 2 ] * 0.5
    end
    
    MOAISim.openWindow ( "Game", screenWidth, screenHeight )
    local viewport =  MOAIViewport.new ()
    viewport:setSize ( 0, 0, screenWidth, screenHeight )
    
    local deviceAspect = screenWidth / screenHeight
    local gameAspect = SCREEN_SIZE [ 1 ] / SCREEN_SIZE [ 2 ]
    local scaleX, scaleY = SCREEN_SIZE[1], SCREEN_SIZE[2]
    
    if deviceAspect < gameAspect then
        scaleX = SCREEN_SIZE[1]
        scaleY = SCREEN_SIZE[1] / deviceAspect
        
        local screen_resize_h = scaleY - SCREEN_SIZE[2];
        
        TOP_PANEL_OFFSET_Y = ( screen_resize_h ) / 2
        RESIZE_SCALE_Y = 1 + ( SCREEN_SIZE[2] / scaleY )
        
    end
    
    if deviceAspect > gameAspect then
        scaleX = SCREEN_SIZE[2] * deviceAspect
        scaleY = SCREEN_SIZE[2]
        
        local screen_resize_w = scaleX - SCREEN_SIZE[1];
        
        TOP_PANEL_OFFSET_X = ( screen_resize_w ) / 2
        RESIZE_SCALE_X =  scaleX / SCREEN_SIZE[1]
    end
    
    MODEL_SCREEN_W, MODEL_SCREEN_H = scaleX, scaleY
    viewport:setScale ( MODEL_SCREEN_W, MODEL_SCREEN_H )
    
    return viewport
end



function main ()
    local viewport = setupViewport ()
    
    -- load resources
    require "res.resources"
    
    gui:init ( viewport )
    av:init
    {
        sampleRate = 44100;
    }

    local controller = AppCtrl.new ()

    app.queue:postEvent ( nil, E_SYSTEM.START, { })
end

main ()

