
E_NET = Event.declare "net"
:types
{
    SESSION_STARTED = 0,
    READY = 1,
    START = 2,
    FR_SCORE = 3,
    FR_FINISH = 4,
    CHEAT_ALLWORDS = 5,
    START_ROUND = 6,
    VIDEO_STATE_CHANGED = 7,
	PAUSE = 8,
	RESUME = 9,
}

AppCtrl = core.class ( Object )

function AppCtrl:ctor ()
    local fsm = FSM.new ( self )

    self.fsm = fsm
    self.model = Model.new ()
    self.netGameSeed = LCG.makeSeed ()
    self.players = {}
    self.friendReady = false
    self.selfReady = false

    app.queue:subscribe ( fsm, FSM.handleEvent, E_SYSTEM )
    app.queue:subscribe ( fsm, FSM.handleEvent, E_TIMEOUT )
    app.queue:subscribe ( fsm, FSM.handleEvent, E_NET )
    app.queue:subscribe ( fsm, FSM.handleEvent, nil, self.model )
    app.queue:subscribe ( fsm, FSM.handleEvent, gui.E_ACTION )
    app.queue:subscribe ( fsm, FSM.handleEvent, GameView.E_ANIMATED )
    app.queue:subscribe ( fsm, FSM.handleEvent, GameView.E_ENABLE_VIDEO_DLG )
    app.queue:subscribe ( fsm, FSM.handleEvent, GameView.E_TURN_CAMERA_ON )

    app.queue:subscribe ( fsm, FSM.handleEvent, E_ROUND_TIMER.TIME_TIC )
    app.queue:subscribe ( self, self.onResume, E_SYSTEM.RESUME )
    app.queue:subscribe ( self, self.onPause, E_SYSTEM.PAUSE )
	app.queue:subscribe ( self, self.onFriendPause, E_NET.PAUSE )
	app.queue:subscribe ( self, self.onFriendResume, E_NET.RESUME )
	
    app.queue:subscribe ( self, self.onIncomingVideoStateChanged, E_NET.VIDEO_STATE_CHANGED )

    fsm:declare
    {
        {
            leave = function ( self )

                self.isMultiplayer = false
                -- Cafe.StartNetSession ( "Tango-Game" )
            end;

            [ E_SYSTEM.START ] = function ( self, sender, event, args )
                return "SPLASH"
            end;
        };

        SPLASH =
            {
                enter = function ( self )
                    print ( "-> SPLASH" )
                    -- show splash dialog

                    self.gameView = GameView.new ( self.model )
                    self.gameView:show ()

                    local isNetworkConnected = false
                    self.gameView:startSplash ( isNetworkConnected )
                    --  self.gameView:splashShowWaitMsg ()

                    self.channel = NetChannel.new ()

                    if self.channel:open () then
                        self.channel.opened:connect ( self,
                                                      function ( self, peerId )
                                                          app.queue:postEvent ( self, E_NET.SESSION_STARTED,
                                                                                {
                                                                                    id = MOAIEnvironment.generateGUID (),
                                                                                    peerId = peerId,
                                                                                }
                                                          )
                                                      end
                        )
                        -- set up observers
                        self.channel.msgReceived:connect ( self,
                                                           function ( self, msgType, msgArgs )
                                                               app.queue:postEvent ( self, E_NET [ msgType ], msgArgs )
                                                           end
                        )
                    end
                    self._connectTimer = Timer.new ( 5 ):start ( 1 )
                    self._waitingPlayersCnt = 2
                end;

                leave = function ( self )
                    if self._connectTimer ~= nil then
                        self._connectTimer:stop ()
                    end
                    self._connectTimer = nil

                    if self._waitingTimer ~= nil then
                        self._waitingTimer:stop ()
                    end

                    self._waitingTimer = nil
                    self._waitingPlayersCnt = nil

                    self.gameView:hideSplash ()
                end;

                [ E_NET.SESSION_STARTED ] = function ( self, sender, event, args )
                    self.session = args
                    peerId = args.peerId
                    self.gameView:splashShowWaitMsg ()
                end;

                [ E_NET.READY ] = function ( self, sender, event, args )
                    print "** ready **"
                    local thisId = self.session.id
                    local thatId = args.id
                    local isHost = thisId < thatId
                    self.session.isHost = isHost

                    if isHost then
                        self.seed = self.netGameSeed
                        print ( "I'm host.", self.seed )
                    end

                    self._waitingPlayersCnt = self._waitingPlayersCnt - 1
                    if self._waitingPlayersCnt == 0 then
                        self.channel:send ( "START", { seed = isHost and self.seed or nil; })
                    end
                end;

                [ E_NET.START ] = function ( self, sender, event, args )
                    self.isMultiplayer = true
                    if not self.session.isHost then
                        self.seed = args.seed
                        print ( "I'm not host.", self.seed )
                    end
                    return "SETUP"
                end;

                [ GameView.E_ANIMATED.SPLASH_MOVE_END ] = function ( self, sender, event, args )


                end;

                [ E_TIMEOUT ] = function ( self, sender, event, args )
                    if sender == self._connectTimer then
                        self._connectTimer = nil

                        if not self.session then
                            return "SETUP"
                        end

                        self.channel:send ( "READY", { id = self.session.id })
                        self._waitingTimer = Timer.new ( 10.0, sendReady )

                        self._waitingPlayersCnt = self._waitingPlayersCnt - 1
                        if self._waitingPlayersCnt == 0 then
                            self.channel:send ( "START", { seed = self.session.isHost and self.seed or nil; })
                        end
                    end

                    if sender == self._waitingTimer then
                        self.isMultiplayer = false
                        return "SETUP"
                    end
                end;
            };

        SETUP =
            {
                enter = function ( self )
                    print "-> SETUP"
                    local videoStatus = CameraCtrl.videoSupportStatus ()
                    if videoStatus > CameraCtrl.DEVICE_TOO_OLD then
                        self.vsc = CameraCtrl.getVsc ()
                    end
                    self.gameView:updateOutgoingVideo ( false )

                    self.gameView:updateIncomingVideo ( false )
                    self.gameView:updateVideoPlaceholders ( videoStatus )
                    return "GAME_SETUP"
                end;

                [ E_NET.START_ROUND ] = function ( self, sender, event, args )
                    self.friendReady = true
                end;

                leave = function ( self )
                end;
            };

        GAME_SETUP =
            {
                enter = function ( self )
                    print "-> GAME_SETUP"

                    if not self.isMultiplayer then
                        self.seed = self.netGameSeed
                    end

                    local model = self.model

                    model.isMultiplayer = self.isMultiplayer
                    Timer.new ( 0.75,
                                function ()
                                    self.gameView:startGame ()
                                end
                    ):start ( 1 )
                end;

                [ E_NET.START_ROUND ] = function ( self, sender, event, args )
                    self.friendReady = true
                end;

                [ GameView.E_ANIMATED.START_GAME ] = function ( self, sender, event, args )
                    self.model:setup ()
                    return "GAME_READY"
                end;

            };

        GAME_READY =
            {
                enter = function ( self )
                    print "-> GAME_READY"

                    return "GAME"
                end;

                [ E_NET.START_ROUND ] = function ( self, sender, event, args )
                    self.friendReady = true
                end;

            };

        GAME =
            {
                enter = function ( self )
                    print "-> GAME"
                    LCG.init ( self.seed )
                    self.model:start ( self.model:getRandMove () )
                end;

                [ E_MODEL.STATE_CHANGED ] = function ( self, sender, event, args )
                    local model = sender
                    if args.nextState == model.STATE_TURN_BEG then
                        print("C= STATE_TURN_BEG")
                    end
                    if args.nextState == model.STATE_TURN then
                        print("C= STATE_TURN")
                        self.friendReady = false
                        self.selfReady = false
                    end
                    if args.nextState == model.STATE_TURN_COMPLETE then
                        print("C= STATE_TURN_COMPLETE")
                    end
                    if args.nextState == model.STATE_TURN_RESULT then
                        print("C= STATE_TURN_RESULT")
                        if self.isMultiplayer  then
                            self.channel:send ( "FR_SCORE", { score = self.model._score } )
                        else
                        end
                        self._resultTimer = Timer.new ( 3 ):start ( 1 )
                    end

                    if args.nextState == model.STATE_TURN_END then
                        print("C= STATE_TURN_END")
                        -- self.seed = self.model:getRandMove ()
                        self.model._friendFinish = false
                    end
                end;


                [ E_NET.START_ROUND ] = function ( self, sender, event, args )
                    self.friendReady = true

                    --  print("Gor Seed:", args.seed )

                    -- if args.seed ~= nil then
                    --     self.seed = args.seed
                    -- end

                    if self.selfReady then
                        self.model:startTurn ( self.model:getRandMove () )
                    end
                    print("START_ROUND")
                end;


                [ E_NET.FR_SCORE ] = function ( self, sender, event, args )
                    self.model._friendScore = args.score
                    self.model._friendFinish = true
                    if self._resultTimer == nil and self.model.state == Model.STATE_TURN_RESULT then
                        print("OOOfinish 1");
                        self.model:finishTurn ()
                    end
                end;

                [ E_TIMEOUT ] = function ( self, sender, event, args )
                    if sender == self._resultTimer then
                        self._resultTimer = nil
                        if self.isMultiplayer then
                            if self.model._friendFinish then
                                print("OOOfinish 2");
                                self.model:finishTurn ()
                            else
                                self.gameView:showWaitForFriends ()
                            end
                        else
                            print("OOOfinish 3");
                            self.model:finishTurn ()
                        end
                    end
                end;

                [ GameView.E_TURN_CAMERA_ON ] = function ( self )
                    CameraCtrl.enableOutgoingVideo ()
                    self:updateOutgoingVideo ( true )
                    self.channel:send ( "VIDEO_STATE_CHANGED", { state = true })
                end;

                [ GameView.E_ENABLE_VIDEO_DLG ] = function ( self, sender, event, args )
                    self.enableVideoDlg = EnableVideoDialog.new ( )
                    self.enableVideoDlg:show ()
                end;

                [ gui.E_ACTION ] = function ( self, sender, event, args )
                    --     print ( "action", args.id )
                    if args.id == "GameView.btnBottom" then
                        if self.model.state == Model.STATE_TURN_BEG and not self.selfReady  then
                            av:playSound("art/sounds/click.ogg")
                            self.selfReady = true;
                            if self.isMultiplayer  then
                                self.channel:send ( "START_ROUND", {} )

                                if self.friendReady then
                                    self.model:startTurn ( self.model:getRandMove () )
                                else

                                    self.gameView:showWaitForFriends ()
                                end
                            else
                                self.model:startTurn ( self.model:getRandMove () )
                            end
                        elseif self.model.state == Model.STATE_DONE then
                            app.queue:postEvent ( self, gui.E_ACTION, { id = "GameOverDialog.Ok" })
                        end
                    end


                    if args.id == "GameView.btnExit" then
                        --return "FINAL"
                        self.gameExitView = ExitDialog.new ( )
                        self.gameExitView:show ()
                        av:playSound("art/sounds/click.ogg")
                    end

                    if args.id == "GameView.btnInfo" then
                        self.gameInfoView = InfoDialog.new ( )
                        self.gameInfoView:show ()
                        av:playSound("art/sounds/click.ogg")
                    end

                    if args.id == "ExitDialog.Yes" then
                        --  av:playSound("art/sounds/click.ogg")
                        self.gameExitView:hide ()
                        self.gameExitView = nil
                        return "FINAL"
                    end

                    if args.id == "EnableVideoDialog.Yes" and self.enableVideoDlg then
                        local dlg = self.enableVideoDlg
                        self.enableVideoDlg = nil
                        dlg:hide()
                        app.queue:postEvent ( self, GameView.E_TURN_CAMERA_ON )
                    end

                    if args.id == "EnableVideoDialog.No" and self.enableVideoDlg then
                        local dlg = self.enableVideoDlg
                        self.enableVideoDlg = nil
                        dlg:hide()
                        av:playSound("art/sounds/click.ogg")
                    end

                    if args.id == "ExitDialog.No" and self.gameExitView ~= nil  then
                        self.gameExitView:hide ()
                        self.gameExitView = nil
                        av:playSound("art/sounds/click.ogg")
                    end
                    if args.id == "GameOverDialog.Ok" then
                        return "FINAL"
                    end

                    if args.id == "InfoDialog.Ok" and self.gameInfoView ~= nil then
                        self.gameInfoView:hide ()
                        self.gameInfoView = nil
                        av:playSound("art/sounds/click.ogg")
                    end

                end;

            };

        FINAL =
            {
                enter = function ( self )

                    -- if tango and self.call and self.vsc then
                    --     self.vsc:stopSending ( )
                    -- end

                    if MOAISim.exit then
                        MOAISim.exit ()
                    else
                        os.exit ()
                    end
                end;
            };
    }
end

function AppCtrl:updateOutgoingVideo ( )
    self.gameView:updateOutgoingVideo ( true )
end

function AppCtrl:updateIncomingVideo ( state )
    self.gameView:updateIncomingVideo ( state )
end

function AppCtrl:onIncomingVideoStateChanged ( sender, event, args )
	local videoStatus = CameraCtrl.videoSupportStatus ()
	local isVideoSupported = videoStatus >= CameraCtrl.VIDEO_SUPPORTED
  local hasVideoPlayback = args.state and isVideoSupported
  self.hadVideoPlayback = hasVideoPlayback
    self.gameView:updateIncomingVideo ( hasVideoPlayback )
end

function AppCtrl:onPause ()
	if self.channel then
		self.channel:send ( "PAUSE", {} )
	end
end

function AppCtrl:onResume ()
	if self.channel then
		self.channel:send ( "RESUME", {} )
	end
	self.gameView:restoreShader ()
end

function AppCtrl:onFriendPause ()
	self:updateIncomingVideo ( false )
end

function AppCtrl:onFriendResume ()
	self:updateIncomingVideo ( self.hadVideoPlayback )
end

------------------------------------------------------------------------------------
