local TIMEOUT = 1

local DIGIT_PIC_PATH = "art/textures/ww_dial_numbers.png"

E_ROUND_TIMER = Event.declare "RoundTimer"
    :types
    {
        TIME_OUT = 1,
        TIME_TIC = 2,
    }

GameTimer = core.class ( gui.Widget )
 
function GameTimer:ctor ( rsrc, context )
    gui.Widget.ctor ( self, rsrc, context )
    
    self.totalSeconds = 90
 --   self.totalSeconds = 12
    self.sendNotify = true
    
    if MOAISim.setPauseCallback then
        MOAISim.setPauseCallback ( 
            function ()
                self.pauseTime = MOAISim.getDeviceTime ()
            end
            )
    end
    
    if MOAISim.setResumeCallback then
        MOAISim.setResumeCallback (
            function ()
                if self.pauseTime ~= nil then
                    self.totalSeconds = self.totalSeconds - ( MOAISim.getDeviceTime () - self.pauseTime )
                    self.pauseTime = nil
                end
            end
            )
    end
    
    self.timer = Timer.new ( 1.0,
        function ()
            self.totalSeconds = self.totalSeconds - 1
            
            if self.totalSeconds <= 0 then
                self.totalSeconds = 0
                if self.sendNotify then
                    self.sendNotify =  false
                    app.queue:postEvent ( self, E_ROUND_TIMER.TIME_OUT, {} )
                end
                self.timer:stop ()
            end
            
            app.queue:postEvent ( self, E_ROUND_TIMER.TIME_TIC, { time = self.totalSeconds } )
            
    
            local minValue = math.floor ( self.totalSeconds / 60 )
            local secValue = self.totalSeconds % 60
            local secValue1 = math.floor ( secValue / 10 )
            local secValue2 = secValue % 10

            self.min.prop:setIndex ( minValue + 1 )
            self.sec1.prop:setIndex ( secValue1 + 1 )
            self.sec2.prop:setIndex ( secValue2 + 1 )
            
        end
    )
    
    self.min = gui.Widget.new
    {
        x = -20, y = 0;
        index = 2;
        scale = 2.0;
        deck = rm:rsrc ( gui.TileDeck ) { texture = DIGIT_PIC_PATH, rows = 3, cols = 4; },
    }:attach ( self )

    self.semicolon = gui.Widget.new
    {
        x = 5, y = 0;
        index = 11;
        scale = 2.0;
        deck = rm:rsrc ( gui.TileDeck ) { texture = DIGIT_PIC_PATH, rows = 3, cols = 4; },
    }:attach ( self )
    
    self.sec1 = gui.Widget.new
    {
        x = 25, y = 0;
        index = 4;
         scale = 2.0;
       deck = rm:rsrc ( gui.TileDeck ) { texture = DIGIT_PIC_PATH, rows = 3, cols = 4; },
    }:attach ( self )

    self.sec2 = gui.Widget.new
    {
        x = 50, y = 0;
        index = 1;
        scale = 2.0;
        deck = rm:rsrc ( gui.TileDeck ) { texture = DIGIT_PIC_PATH, rows = 3, cols = 4; },
    }:attach ( self )
end

function GameTimer:start ( loopsCnt )
    if loopsCnt == nil then
        loopsCnt = self.totalSeconds * 5
    end
    self.timer:start ( loopsCnt )    
end
  
function GameTimer:stop ()
   -- totalSeconds = 0
    self.timer:stop ()
end



function GameTimer:hide( )
    self.min.prop:seekColor ( 0, 0, 0, 0.0, 0.0)
    self.semicolon.prop:seekColor ( 0, 0, 0, 0.0, 0.0)
    self.sec1.prop:seekColor ( 0, 0, 0, 0.0, 0.0)
    self.sec2.prop:seekColor ( 0, 0, 0, 0.0, 0.0)
end

function GameTimer:show( time )
    self.min.prop:seekColor ( 1, 1, 1, 1, time)
    self.semicolon.prop:seekColor ( 1, 1, 1, 1, time)
    self.sec1.prop:seekColor ( 1, 1, 1, 1, time)
    self.sec2.prop:seekColor ( 1, 1, 1, 1, time)
end



function GameTimer:addCheatTime ()
   self.totalSeconds = self.totalSeconds + 120 
end






