GameView = core.class ( gui.Scene )
{
    E_ANIMATED = Event.declare "GameView.animated"
    :types
    {
        START_GAME      = 0,
        MOVE            = 1,
        SPLASH_DONE     = 2,
        BALL_HIDE       = 3,
        BALL_SHOW       = 4,
        THIMBLE_MOVE    = 5,
        EMPTY_THIMBLE_SHOW = 6,
        SPLASH_MOVE_END = 7,
    };

    E_ENABLE_VIDEO_DLG = Event.declare "GameView.EnableVideoDialog";
    E_TURN_CAMERA_ON = Event.declare "GameView.TurnCameraOn";
}

function GameView:ctor ( model )
    local root = rm.RSRC_GAME_VIEW:create ()
    gui.Scene.ctor ( self, root )
    self.model = model

    self._thimble = {}

    for i = 1, 3 do
        local tm = Thimble.new
        {
            x = -200 + 200 * ( i - 1 ), y = -210;
            scale = 1.0,
        }:attach ( self.root )
        tm:setModel(model);
        table.insert(self._thimble, tm)
    end

    self._inputListeners =
        {
            self,
            root.btnBottom,
            root.btnExit,
            root.btnInfo,
        }


    if MOAIVideoTexture then


        local RSRC_VIDEO_PREVIEW = rm:rsrc ( gui.Widget )
        {
            deck = rm:rsrc ( gui.GfxQuad )
            {
                rect = { 11, -43, 242, -273 },
                uvRect = { 0, 0, 1, 1 },

                texture = rm:rsrc ( gui.MultiTexture )
                {
                    "art/textures/video_preview_mask.png",
                    rm:rsrc ( gui.VideoTexture )
                    {
                        type = MOAIVideoTexture.PREVIEW,
                    },
                }

            },
            shader = self:_createShader ( "PreviewShader" ),
        }

        self.video_prev = RSRC_VIDEO_PREVIEW:create ()
        :attach ( root.previewBackground.video )


        local RSRC_VIDEO_PLAYBACK = rm:rsrc ( gui.Widget )
        {
            deck = rm:rsrc ( gui.GfxQuad )
            {
                --	rect = { -299, -6, -6, -378 },
                rect = { 18, -65, 427, -494 },
                uvRect = { 1, 0, 0, 1 },

                texture = rm:rsrc ( gui.MultiTexture )
                {
                    "art/textures/video_playback_mask.png",
                    rm:rsrc ( gui.VideoTexture )
                    {
                        type = MOAIVideoTexture.PLAYBACK,
                    },
                }
            },
            shader = self:_createShader ( "PlaybackShader" ),
        }

        self.video_playback = RSRC_VIDEO_PLAYBACK:create ()
        :attach ( root.playbackBackground.video )
    end

    self.moveSelected = core.signal ()

    app.queue:subscribe ( self, self.onModelEvent, E_MODEL, model )
    app.queue:subscribe ( self, self.onAction, gui.E_ACTION )
    app.queue:subscribe ( self, self.onAnimationEnd, GameView.E_ANIMATED )

    self:setStatusText ( "<45>TAP TO START GAME</45>", true, false )
    self.root.txtRound:setText (string.format ("<c:28C4DEFF>%d/%d</>", 1, self.model.MAX_ROUNDS))
    self.inputBlocked = 0
end


function GameView:restoreShader ()
    if self.video_prev then
        local shader = self:_createShader ( "PreviewShader" )
        self.video_prev.rsrc.shader = shader;
        self.video_prev.prop:setShader ( shader )
    end

    if self.video_playback then
        shader = self:_createShader ( "PlaybackShader" )
        self.video_playback.rsrc.shader = shader;
        self.video_playback.prop:setShader ( shader )
    end
end

function GameView:_createShader ( name )
    local file

    file = assert ( io.open ( string.format ( "src/view/shaders/%s.vsh", name ) ) )
    local vsh = file:read ( "*all" )
    file:close ()

    file = assert ( io.open ( string.format ( "src/view/shaders/%s.fsh", name ) ) )
    local fsh = file:read ( "*all" )
    file:close ()

    local shader = MOAIShader.new ()

    shader:reserveUniforms ( 2 )
    shader:declareUniformSampler ( 1, "sampler0", 1 )
    shader:declareUniformSampler ( 2, "sampler1", 2 )

    shader:setVertexAttribute ( 1, "position" )
    shader:setVertexAttribute ( 2, "uv" )
    shader:setVertexAttribute ( 3, "color" )
    shader:load ( vsh, fsh )

    return shader
end

function GameView:getInputListeners ()
    if self.inputBlocked == 0 then
        return self._inputListeners
    end
end

function GameView:onViewportChanged ( sender, event, args )
    -- adjust subviews
end

function GameView:startSplash ( isMultiplayer )
    self.splashScreen = SplashScreen.new (
        {
            x = 0, y = 0;
        },
        nil,
        "words" )
    :attach ( self.root )

    --  av:playSound("art/sounds/shuffle_s.ogg")

    -- self:hideControl ( self.root.scoreInfoYou )
    -- self:hideControl ( self.root.scoreInfoFriend )

end

function GameView:splashShowWaitMsg ()
    if self.splashScreen ~= nil  then
        self.splashScreen:showWaitMsg ()
    end
end

function GameView:hideControl ( ctrl )
    local xLoc, yLoc  = ctrl.transform:getLoc ();
    ctrl.transform:setLoc(xLoc, yLoc + 1200 );
end

function GameView:showControl ( ctrl )
    local xLoc, yLoc  = ctrl.transform:getLoc ();
    ctrl.transform:setLoc(xLoc, yLoc - 1200 );
end


function GameView:hideSplash ( )
    if self.splashScreen ~= nil then
        self.splashScreen:hide ()
    end
end


function GameView:startGame ()
    app.queue:postEvent ( self, self.E_ANIMATED.START_GAME, {})
    local status = CameraCtrl.videoSupportStatus()
    if not self.model.isMultiplayer and status >= CameraCtrl.VIDEO_SUPPORTED then
        self:updateVideoPlaceholders ( CameraCtrl.NOT_ON_CALL )
    else
        if status == CameraCtrl.VIDEO_SUPPORTED then
            app.queue:postEvent ( self, self.E_ENABLE_VIDEO_DLG, {} )
        elseif status == CameraCtrl.VIDEO_LIVE then
            app.queue:postEvent ( self, self.E_TURN_CAMERA_ON )
        end
    end
end

function GameView:setInteractive ( interactive )
    self.isInteractive = interactive
end

function GameView:onTouchBegan ( sender, event, args )
    local x, y = args.x, args.y

    if self.model.state == Model.STATE_TURN_WAIT_SELECT then
        for i = 1,3 do
            if self._thimble[i]:acceptsTouch ( x, y) then
		--	print ("On Touch",i)
                self.model:selectThimble (i)
            end
        end
    end

    if self.model.state == Model.STATE_DONE then
        app.queue:postEvent ( self, gui.E_ACTION, { id = "GameOverDialog.Ok" })
    elseif self.model.state == Model.STATE_TURN_BEG then
        app.queue:postEvent ( self, gui.E_ACTION, { id = "GameView.btnBottom" })
    end

    if self.inputBlocked > 0 then
        return
    end

    if not self.isInteractive then
        return
    end

end

function GameView:onAnimationEnd ( sender, event, args  )

    if event == GameView.E_ANIMATED.SPLASH_DONE then
        print("ON HIDE")
        if self.splashScreen ~= nil then
            self.splashScreen:detach ()
            self.splashScreen = nil
        end
        self._thimble[self.model._ballPos]:showBall ( true, function ()
                                                          self.model:_setState ( self.model.STATE_TURN_BEG )
                                                          self:setStatusText ( "<45>TAP TO START GAME</>", true )
                                                            end )
    end

    if event == GameView.E_ANIMATED.BALL_HIDE then
        self:moveAnimation ()
    end

    if event == GameView.E_ANIMATED.BALL_SHOW then

    end

    if event == GameView.E_ANIMATED.EMPTY_THIMBLE_SHOW then
        self._thimble[self.model._ballPos]:showBall ( true )
    end

    if event == GameView.E_ANIMATED.THIMBLE_MOVE then
        if self.splashScreen ~= nil then
            return
        end

        self._MoveAniCount = self._MoveAniCount - 1
        if self._MoveAniCount ~= 0 then
            return
        end

        self._thimble[1].transform:setLoc ( -200, -210 );
        self._thimble[2].transform:setLoc ( 0, -210 );
        self._thimble[3].transform:setLoc ( 200, -210 );
        local shuffle_count = 23 + ( self.model._round - 1 ) * 7
        if self._CurMove >= shuffle_count then
            self.model:waitSelection ()
        else
            self:NextMoveAnimation ()
        end
    end

end

function GameView:onAction ( sender, event, args )

    print ( "View action", args.id )

    if self.splashScreen ~= nil then
        return
    end

    if self.model.state == self.model.STATE_DONE or self.model.state == self.model.STATE_INIT then
        return
    end



end

function GameView:onModelEvent ( sender, event, args )
    local model = sender
    if event == E_MODEL.MOVE_MADE then
        local function onComplete ()
            app.queue:postEvent ( self, self.E_ANIMATED.MOVE, {} )
        end
        onComplete ()
    elseif event == E_MODEL.FRIEND_MOVES then
        self.friendScore:setScore (self.model:getFriendScore ())
    elseif event == E_MODEL.STATE_CHANGED then
        if args.prevState == model.STATE_INIT then

	end

        if args.nextState == model.STATE_TURN_BEG then
            self.root.txtRound:setText (string.format ("<c:28C4DEFF>%d/%d</>", model._round, self.model.MAX_ROUNDS))
            if model._round ~= 1 then
                self:setStatusText ("<38>TAP TO START NEXT ROUND</>", true)
            end
            self:updateScore ()
        end

        if args.nextState == model.STATE_TURN then
            self:setStatusText ("FOLLOW THE BALL", false, true)
            self:countDown ()
            Timer.new ( 3, function ()
                            self:setStatusText ("FOLLOW THE BALL")
                            self._thimble[model._ballPos]:hideBall ( true )
                           end ):start ( 1 )
        end

        if args.nextState == model.STATE_TURN_WAIT_SELECT then
            self:setStatusText ("MAKE A CHOICE")
            for _, thimble in ipairs ( self._thimble ) do
                thimble:startHighlight ()
            end
        end

        if args.nextState == model.STATE_TURN_RESULT then
            for _, thimble in ipairs ( self._thimble ) do
                thimble:stopHighlight ()
            end
            if model._roundWin then
                self:setStatusText ("RIGHT!")
                av:playSound("art/sounds/chimes.ogg")
                self._thimble[model._ballPos]:showBall ( true )
            else
                self:setStatusText ("MISS!")
                av:playSound("art/sounds/fail.ogg")
                self._thimble[model._userSelection]:showEmptyThimble ( )
            end
            self:updateScore ()
        end

        if args.nextState == model.STATE_TURN_END then
            model:nextTurn()
        end

        if args.nextState == model.STATE_DONE then
            self:gameOver ()
        end
    end
end

function GameView:countDown ()
    local go = rm:rsrc ( gui.Widget )
    {
        x = 7, y =-210;

        deck = rm:rsrc ( gui.GfxQuad ) {
            scaleX = 0.5, scaleY = 3;
            texture = "art/textures/yellow_spot.png" },

        rm:rsrc ( gui.ShadowTextBox, "/txtResult" )
        {
            font = "art/fonts/futurica80.fnt",
            color = { 1, 1, 1, 1 },
            fontSize = 105,
            rect = {-240, -300, 240, 300,},
            hAlignment = MOAITextBox.CENTER_JUSTIFY, vAlignment = MOAITextBox.CENTER_JUSTIFY;
            YFlip = true,
        } }:create ()
    go:attach ( self.root )

    local action = gui.TimelineAction.new
    {
        duration = 2.0;
        loopsCnt = 0;

        { t = 0.05; function () go.txtResult:setText ( "<c:28C4DEFF>3</c>" ) end };
        { t = 1.05; function () go.txtResult:setText ( "<c:28C4DEFF>2</c>" ) end };
        { t = 2.05; function () go.txtResult:setText ( "<c:28C4DEFF>1</c>" ) end };
        { t = 3.05; function () go:detach () end };
    } :start()
end

function GameView:gameOver ()
    local model = self.model
    local deckscale = 1
    if model.isMultiplayer and model._score == model._friendScore then
        deckscale = 0.5
    end

    local go = rm:rsrc ( gui.Widget )
    {
        x = 0, y =-210;

        rm:rsrc ( gui.Widget )
        {
            deck = rm:rsrc ( gui.GfxQuad ) {
                scaleX = 1, scaleY = 2;
                texture = "art/textures/yellow_spot.png" },
        },

        rm:rsrc ( gui.ShadowTextBox, "/txtResult" )
        {
            y = 10;
            font = "art/fonts/futurica80.fnt",
            color = { 1, 1, 1, 1},
            fontSize = 38,
            rect = {-240, -300, 240, 300,},
            hAlignment = MOAITextBox.CENTER_JUSTIFY, vAlignment = MOAITextBox.CENTER_JUSTIFY;
            YFlip = true,
        };

        rm:rsrc ( gui.ShadowTextBox, "/txtScore" )
        {
            y = -25;
            font = "art/fonts/futurica80.fnt",
            color = { 1, 1, 1, 1},
            fontSize = 25,
            shadowIntensity = 0,
            rect = {-240, -300, 240, 300,},
            hAlignment = MOAITextBox.CENTER_JUSTIFY, vAlignment = MOAITextBox.CENTER_JUSTIFY;
            YFlip = true,
        }
    }:create ()
    go:attach ( self.root )

    if model.isMultiplayer then
        if model._score == model._friendScore then
            go.txtResult:setText ( "<c:28C4DEFF>TIE</c>" )
            av:playSound ( "art/sounds/win.ogg" )
        elseif model._score > model._friendScore then
            go.txtResult:setText ( "<c:28C4DEFF>YOU WIN!</>" )
            av:playSound ( "art/sounds/win.ogg" )
        else
            go.txtResult:setText ( "<c:ff0000ff>YOU LOSE</>" )
            av:playSound ( "art/sounds/lose.ogg" )
        end
        go.txtScore:setText ( "<c:0>" .. model._score .. " TO " .. model._friendScore .. "</c>" )
        av:playSound( model._score < model._friendScore and "art/sounds/lose.ogg" or "art/sounds/win.ogg" )
    else
        if model._score == model.MAX_ROUNDS then
            go.txtResult:setText ("<c:28C4DEFF>PERFECT!</c>")
        else
            go.txtResult:setText ("<c:28C4DEFF>GAME OVER</c>")
        end
        go.txtScore:setText ( "<c:0>" .. model._score .. " OUT OF " .. model.MAX_ROUNDS .. "</c>" )
        av:playSound("art/sounds/win.ogg")
    end

    local curve = MOAIAnimCurve.new ()
    curve:reserveKeys ( 4 )
    curve:setKey ( 1, 0.00, 0.1 )
    curve:setKey ( 2, 0.50, 2.0 )
    curve:setKey ( 3, 0.75, 1.8 )
    curve:setKey ( 4, 1.00, 2.0 )

    local anim = MOAIAnim.new ()
    anim:reserveLinks ( 2 )
    anim:setLink ( 1, curve, go.transform, MOAITransform.ATTR_X_SCL, MOAIEaseType.EASE_OUT )
    anim:setLink ( 2, curve, go.transform, MOAITransform.ATTR_Y_SCL, MOAIEaseType.EASE_OUT )

    anim:setListener ( MOAIAction.EVENT_STOP, onStop )

    anim:start ()

    self:updateScore ()
    self:showTapToExit ( )
end

function GameView:moveAnimation()
    --	av:playSound("art/sounds/slide.ogg")
    self._CurMove = 0
    self:NextMoveAnimation ()
end

function GameView:NextMoveAnimation()
    self._CurMove = self._CurMove + 1
    self._MoveAniCount = 2
    --   print("Mov",self._CurMove)

    local move_type = self.model._movies[self._CurMove]

    if move_type == 1 then
        self._thimble[1]:moveTo(-200, 0, 2 )
        self._thimble[2]:moveTo(0, -200, 1 )
        self.model:swapThimble ( 1, 2 )
    end
    if move_type == 2 then
        self._thimble[2]:moveTo(0, 200, 2 )
        self._thimble[3]:moveTo(200, 0, 1 )
        self.model:swapThimble ( 2, 3 )
    end
    av:playSound("art/sounds/slide.ogg")
end

function GameView:showWaitForFriends ()
    self:setStatusText ("WAITING FOR FRIEND", false, true)
end

function GameView:clearVideo ()

    if self.video_playback ~= nil then
        self.video_playback:detach ()
    end

    if self.video_prev ~= nil then
        self.video_prev:detach ()
    end
end

function GameView:setStatusText ( text, isActive, isPulsating )
    if isPulsating == nil then
        isPulsating = isActive
    end

    if isActive then
        self.root.txtStatus:setText ( "<c:000000FF>" .. text .. "</>" )
    else
        self.root.txtStatus:setText ( "<c:28C4DEFF>" .. text .. "</>" )
    end

    if isPulsating then
        self.startPulsating ( self.root.txtStatus, 0.05, 0.5 )
    else
        self.stopPulsating ( self.root.txtStatus )
    end
end

function GameView:updateScore ( )
    self.root.txtOwnScore:setText (tostring (self.model._score))
    if self.model.isMultiplayer then
        self.root.txtFriendScore:setText (tostring (self.model._friendScore))
    end
end

function GameView.startPulsating ( view, scale, time )
    if view.isPulsating then
        return
    end
    local scale = scale or 0.05
    local time = time or 0.35
    local zoomIn, zoomOut

    zoomOut = function ()
        local action = view.transform:moveScl ( -scale, -scale, time, MOAIEaseType.SOFT_EASE_OUT )
        action:setListener ( MOAIAction.EVENT_STOP, zoomIn )
    end

    zoomIn = function ()
        if view.isPulsating then
            local action = view.transform:moveScl ( scale, scale, time, MOAIEaseType.SOFT_EASE_OUT)
            action:setListener ( MOAIAction.EVENT_STOP, zoomOut )
        end
    end

    view.isPulsating = true
    zoomIn ()
end

function GameView.stopPulsating ( view )
    view.isPulsating = false
end

function GameView:showTapToExit ( )
    self:hideControl(self.root.txtRoundInfo)
    self:hideControl(self.root.txtRound)

    self:setStatusText ( "TAP TO EXIT", true )
end

function GameView:textZoomOut ( )
    local action = self.textZoomCtrl.transform:moveScl ( -0.05, -0.05, 0.35, MOAIEaseType.SOFT_EASE_OUT )
    action:setListener ( MOAIAction.EVENT_STOP, GameView.textZoomIn )
    action.textZoomCtrl = self.textZoomCtrl
end

function GameView:textZoomIn ( )
    local action = self.textZoomCtrl.transform:moveScl ( 0.05, 0.05, 0.35, MOAIEaseType.SOFT_EASE_OUT)
    action:setListener ( MOAIAction.EVENT_STOP, GameView.textZoomOut )
    action.textZoomCtrl = self.textZoomCtrl
end

function GameView:updateOutgoingVideo ( out_video )
    self.root.previewBackground.transform:setLoc( 57, out_video and 378 + TOP_PANEL_OFFSET_Y or 2500 );
end

function GameView:updateIncomingVideo ( inc_video )
	self.hasVideoPlayback = hasVideo
    self.root.playbackBackground.transform:setLoc( -312, inc_video and 505 + TOP_PANEL_OFFSET_Y or 2500 );
end

function GameView:updateVideoPlaceholders ( video_status )
    if video_status == CameraCtrl.NOT_ON_CALL then
        self.root.txtPlaybackMessage:setText ( "TRY\nTHIS GAME\nIN A CALL\nWITH YOUR\nFRIENDS!" )
        self.root.txtPreviewMessage:setText ( "" )
    elseif video_status == CameraCtrl.HOST_TOO_OLD then
        self.root.txtPlaybackMessage:setText ( "UPDATE\nTANGO\nTO VIEW\nVIDEO\nHERE!" )
        self.root.txtPreviewMessage:setText ( "" )
        Timer.new ( 10, function ()
                        self.root.txtPlaybackMessage.text.prop:seekColor ( 0, 0, 0, 0, 2 )
                        self.root.txtPlaybackMessage.shadow.prop:seekColor ( 0, 0, 0, 0, 2 )
                        end ) :start ( 1 )
    elseif video_status == CameraCtrl.DEVICE_TOO_OLD then
        self.root.txtPlaybackMessage:setText ( "" )
        self.root.txtPreviewMessage:setText ( "DEVICE NOT\nSUPPORTED" )
        Timer.new ( 10, function ()
                        self.root.txtPreviewMessage.text.prop:seekColor ( 0, 0, 0, 0, 2 )
                        self.root.txtPreviewMessage.shadow.prop:seekColor ( 0, 0, 0, 0, 2 )
                        end ) :start ( 1 )
    elseif video_status == CameraCtrl.VIDEO_SUPPORTED then
        self.root.txtPlaybackMessage:setText ( "" )
        self.root.txtPreviewMessage:setText ( "" )
    elseif video_status == CameraCtrl.VIDEO_LIVE then
        self.root.txtPlaybackMessage:setText ( "" )
        self.root.txtPreviewMessage:setText ( "" )
    end
    -- self.root.txtRoundInfo:setText ( tostring ( video_status ) )
end
