WordBoard = core.class ( gui.Widget )

function WordBoard:ctor ( rsrc, context )
    gui.Widget.ctor ( self, rsrc, context )
end

function WordBoard:createBoard( words )
    local GRID_MAX_ROWS = 9
    local GRID_CHAR_DX = 40 
    local GRID_WORD_DX = 10 
    local GRID_WORD_DY = 45
    
    -- count words of a particular length
    local wordsCount = { 0, 0, 0, 0, 0, 0, }

    for _, word in ipairs ( words ) do
        wordsCount [ #word ] = 1 + wordsCount [ #word ]
    end
    
    -- normalize row count  
    local w3and4 = ( wordsCount[3] + wordsCount[4] ) / 2
    local row5and6 = wordsCount[5] + wordsCount[6]
  
    local rowCount = #words / 3

    if rowCount < row5and6 then
        rowCount = row5and6
    end
    if rowCount > GRID_MAX_ROWS then
        rowCount = GRID_MAX_ROWS
    end
 
    -- fill table 
    local curRow = 0
    local curCol = 0
    local columnOffset = 0
    
    self.words = {}
  
    for _, word in ipairs ( words ) do
        local ctrlWord = gui.Widget.new
        {
            x = columnOffset, y = -curRow * GRID_WORD_DY;
        }:attach(self)
    
        for n = 0, #word - 1 do
            gui.Widget.new
            {
                x = n * GRID_CHAR_DX, y = 0;
				scale = 1.0,
                
        		deck = rm:rsrc ( gui.GfxQuadDeck )
                {
                    [ 1 ] = { sprite = "board_word_box.png"; },
                    [ 2 ] = { sprite = "board_word_box_friend.png"; },
                    [ 3 ] = { sprite = "board_word_box_miss.png"; },
                },
                
                
                rm:rsrc ( gui.TextBox, "letter" )
                {
                    x =  0, y = 0;
                    scale = 0.875;
                    font = "art/fonts/hobo_outline.fnt",
                    fontSize = 36,
                    rect = {-100, -150, 100, 150,},
                    text = "",
                    hAlignment = MOAITextBox.CENTER_JUSTIFY, vAlignment = MOAITextBox.CENTER_JUSTIFY;
                    YFlip = true,
                },

                --[[
                rm:rsrc ( gui.Widget, "letter" )
                {
                    x = 0, y = 0; 
                    scale = 0.9;
                    index = 27;
                    deck = rm:rsrc ( gui.TileDeck ) { texture = "art/textures/letters.png", rows = 4, cols = 7; },
                }
                --]]
            }:attach ( ctrlWord )
        end
        self.words[word] = ctrlWord
        
        curRow = curRow + 1
        if curRow >= rowCount then
            curCol = curCol + 1
            curRow = 0
            local lt_count = #word
            if lt_count > 4 then
               lt_count = 4
            end
            columnOffset = columnOffset + lt_count * GRID_CHAR_DX + GRID_WORD_DX
        end
    end
end

function WordBoard:showWord ( word, mark )
    local wordWdg = self.words [ word ]
    if wordWdg == nil then
        return false
    end
    
    local word_up = string.upper(word)
    
    for i, btn in ipairs ( wordWdg:getChildren ()) do
        local letterWdg = btn.letter
       -- print("STR=",string.sub(word,i,i))
        letterWdg.prop:setString ( string.sub( word_up, i, i ))
        if mark then
            btn.prop:setIndex ( 3 )
            letterWdg.prop:setColor ( 249/255, 208/255, 26/255, 1 )
        else
            -- letterWdg.prop:setColor ( 62/255, 118/255, 105/255, 1 )
            letterWdg.prop:setColor ( 29/255, 71/255, 93/255, 1 )
        end
    end
    
    --[[
    for i, btn in ipairs ( wordWdg:getChildren ()) do
        local letterWdg = btn.letter
        letterWdg.prop:setIndex ( string.byte ( word, i ) - string.byte ( "a" ) + 1 )
    end
    --]]
    return true
end

function WordBoard:showWords ( words )
    local count = #words
    for i, word in ipairs ( words ) do
        self:showWord ( word, true )
    end
end

function WordBoard:markFriendWord ( word )
    local wordWdg = self.words [ word ]
    if wordWdg == nil then
        return false
    end
    
    for i, btn in ipairs ( wordWdg:getChildren ()) do
        btn.prop:setIndex ( 2 )
    end   
    
    return true
end
  
  
  