
E_NET = Event.declare "net"
    :types
    {
        SESSION_STARTED = 0,
        READY = 1,
        START = 2,
        FR_SCORE = 3,
        FR_FINISH = 4,
        CHEAT_ALLWORDS = 5,
    }

AppCtrl = core.class ( Object )

function AppCtrl:ctor ()
    local fsm = FSM.new ( self )

    self.fsm = fsm
    self.model = Model.new ()
    self.netGameSeed = self.model:getRandWordKey ()
    self.players = {}
    self._exit_flow = false
    
    self.cheat_counter = 0
    
    app.queue:subscribe ( fsm, FSM.handleEvent, E_SYSTEM )
    app.queue:subscribe ( fsm, FSM.handleEvent, E_TIMEOUT )
    app.queue:subscribe ( fsm, FSM.handleEvent, E_NET )
    app.queue:subscribe ( fsm, FSM.handleEvent, nil, self.model )
    app.queue:subscribe ( fsm, FSM.handleEvent, gui.E_ACTION )
    app.queue:subscribe ( fsm, FSM.handleEvent, GameView.E_ANIMATED )
    
    app.queue:subscribe ( fsm, FSM.handleEvent, E_ROUND_TIMER.TIME_TIC )
    

    fsm:declare
    {
        {
            leave = function ( self )
                
                self.isMultiplayer = false
                -- Cafe.StartNetSession ( "Tango-Game" )
            end;

            [ E_SYSTEM.START ] = function ( self, sender, event, args )
                return "SPLASH"
            end;
        };

        SPLASH =
        {
            enter = function ( self )
                print ( "-> SPLASH" )
                -- show splash dialog
                
                self.gameView = GameView.new ( self.model )
                self.gameView:show ()

                local isNetworkConnected = false -- Cafe.IsNetConnected ()
                self.gameView:startSplash ( isNetworkConnected )
               -- self.gameView:splashShowWaitMsg ()

                self.channel = NetChannel.new ()
                
                if self.channel:open () then
                    self.channel.opened:connect ( self,
                        function ( self, peerId )
                            app.queue:postEvent ( self, E_NET.SESSION_STARTED, 
                                {
                                    id = MOAIEnvironment.generateGUID (),
                                    peerId = peerId,
                                }
                            )
                        end
                    )                   
                -- set up observers
                    self.channel.msgReceived:connect ( self,
                        function ( self, msgType, msgArgs )
                            app.queue:postEvent ( self, E_NET [ msgType ], msgArgs )
                        end
                    )
                end
                self._connectTimer = Timer.new ( 5 ):start ( 1 )
                self._waitingPlayersCnt = 2
            end;

            leave = function ( self )
                if self._connectTimer ~= nil then
                    self._connectTimer:stop ()
                end
                self._connectTimer = nil
                
                if self._waitingTimer ~= nil then
                    self._waitingTimer:stop ()
                end

                self._waitingTimer = nil
                self._waitingPlayersCnt = nil

                self.gameView:hideSplash ()
            end;

            [ E_NET.SESSION_STARTED ] = function ( self, sender, event, args )
                self.session = args
                peerId = args.peerId
                self.gameView:splashShowWaitMsg ()
            end;
            
            [ E_NET.READY ] = function ( self, sender, event, args )
                print "** ready **"
                local thisId = self.session.id
                local thatId = args.id
                local isHost = thisId < thatId
                self.session.isHost = isHost

                if isHost then
                    self.seed = self.netGameSeed
                    print ( "I'm host.", self.seed )
                end

                self._waitingPlayersCnt = self._waitingPlayersCnt - 1
                if self._waitingPlayersCnt == 0 then
                    self.channel:send ( "START", { seed = isHost and self.seed or nil; })
                end
            end;
            
            [ E_NET.START ] = function ( self, sender, event, args )
                self.isMultiplayer = true
                if not self.session.isHost then
                    self.seed = args.seed
                    print ( "I'm not host.", self.seed )
                end
                return "SETUP"
            end;
            
            [ E_TIMEOUT ] = function ( self, sender, event, args )
                if sender == self._connectTimer then
                    self._connectTimer = nil

                    if not self.session then
                        return "SETUP"
                    end

                    self.channel:send ( "READY", { id = self.session.id })
                    self._waitingTimer = Timer.new ( 10.0, sendReady )

                    self._waitingPlayersCnt = self._waitingPlayersCnt - 1
                    if self._waitingPlayersCnt == 0 then
                        self.channel:send ( "START", { seed = self.session.isHost and self.seed or nil; })
                    end
                end
 
                if sender == self._waitingTimer then
                    self.isMultiplayer = false
                    return "SETUP"
                end
            end;            
       };

        SETUP =
        {
            enter = function ( self )
                print "-> SETUP"
                return "GAME_SETUP"
            end;

            leave = function ( self )
            end;
        };

        GAME_SETUP =
        {
            enter = function ( self )
                print "-> GAME_SETUP"

                local model = self.model
                
                model.isMultiplayer = self.isMultiplayer
                Timer.new ( 0.75,
                    function ()
                        self.gameView:startGame ()
                    end
                ):start ( 1 )
            end;

            [ GameView.E_ANIMATED.START_GAME ] = function ( self, sender, event, args )
                self.model:setup ()
                return "GAME_READY"
            end;
        };

        GAME_READY =
        {
            enter = function ( self )
                print "-> GAME_READY"

                return "GAME"
            end;
        };

        GAME =
        {
            enter = function ( self )
                print "-> GAME"
                if not self.isMultiplayer then
                   self.seed = self.model:getRandWordKey ()
                end
                
          --     self.seed = "alexic"
           --    self.seed = "sylvan"
           --    self.seed = "fatwas"
                
                
                self.model:start ( self.seed)
            end;
            
            
            [ E_NET.CHEAT_ALLWORDS ] = function ( self, sender, event, args )
                self.cheat_counter = 4
                self:activateCheatAllWords ()
            end;
            
            [ E_NET.FR_SCORE ] = function ( self, sender, event, args )
                self.model:setFriendScore ( args.score )
                self.gameView.board:markFriendWord ( args.word )
            end;
            
            [ E_NET.FR_FINISH ] = function ( self, sender, event, args )
                self.model:setFriendFinish ( args.score, args.all_words )
            end;

            [ E_MODEL.STATE_CHANGED ] = function ( self, sender, event, args )
                local model = sender
                if args.nextState == model.STATE_TURN_END then
--                    local ctrl = self.players [ self.model.currPlayer ]
--                    ctrl:deactivate ( )
                    model:nextTurn ()
                end
                if args.nextState == model.STATE_TURN_BEG then
--                    local ctrl = self.players [ self.model.currPlayer ]
--                    ctrl:activate ( )
                end
                if args.nextState == model.STATE_DONE then
                    
                    if self.isMultiplayer then
                        self.channel:send ( "FR_FINISH", { score = self.model._score, all_words = self.model:isFindAllWords ()  } )
                    end
                    
                    -- check PopUp
                    if self.gameInfoView or self.gameExitView then
                       self._exit_flow = true
                    else
                        return "GAME_OVER"
                    end
                end
            end;
            
            [ E_MODEL.MOVE_MADE ] = function ( self, sender, event, args )
                if args.correct then
                    if self.isMultiplayer then
                        self.channel:send ( "FR_SCORE", { score = self.model._score, word = args.word } )
                    end
                    av:playSound("art/sounds/correct_gues_s.ogg")
                else
                    av:playSound("art/sounds/incorrect_word_s.ogg")
                end
            end;
            
            [ E_ROUND_TIMER.TIME_TIC ] = function ( self, sender, event, args )
                if args.time <= 10 and args.time ~= 0 then
                    local snd = "art/sounds/tick_s.ogg"
                    av:playSound ( snd )
                    if args.time <= 5 then
                        Timer.new ( 0.5,
                            function ()
                                av:playSound ( snd )
                            end
                        ):start ( 1 )
                    end
                end
            end;
            

            [ gui.E_ACTION ] = function ( self, sender, event, args )
                print ( "action", args.id )
                if args.id == "GameView.Exit" then
                    self.gameExitView = ExitDialog.new ( )
                    self.gameExitView:show () 
                    av:playSound("art/sounds/press_letter_s.ogg")
               end
                if args.id == "GameView.Info" then
                    self.gameInfoView = InfoDialog.new ( )
                    self.gameInfoView:show ()
                    --av:playSound("art/sounds/time_up_s.ogg")
                    av:playSound("art/sounds/press_letter_s.ogg")
                    self.cheat_counter = self.cheat_counter + 1
                end
                
                if args.id == "ExitDialog.Yes" then
                    av:playSound("art/sounds/press_letter_s.ogg")
                    self.gameExitView:hide ()
                    self.gameExitView = nil
                    return "FINAL"
                end
                if args.id == "ExitDialog.No" and self.gameExitView ~= nil  then
                    self.gameExitView:hide ()
                    self.gameExitView = nil
                    av:playSound("art/sounds/press_letter_s.ogg")
                    if self._exit_flow then
                        return "GAME_OVER" 
                    end
               end
                if args.id == "InfoDialog.Ok" and self.gameInfoView ~= nil then
                    self.gameInfoView:hide ()
                    self.gameInfoView = nil
                    av:playSound("art/sounds/press_letter_s.ogg")
                    if self._exit_flow then
                        return "GAME_OVER" 
                    end
                    
                    if self.cheat_counter == 3 then
                        self.channel:send ( "CHEAT_ALLWORDS", { cheat = 1 } )
                        self:activateCheatAllWords ()
                    end
                end
            end;

        };

        -- game over popup
        GAME_OVER =
        {
            enter = function ( self )
                print "-> GAME_OVER"
                self.gameView:clearInput ()
                if self.model:isFindAllWords () then
                    self.gameGameOverView = GameOverDialog.new ( self.model )
                    self.gameGameOverView:show () 
                    if self.isMultiplayer == false then
                        av:playSound("art/sounds/game_win_s.ogg")
                    end
                else
                    av:playSound("art/sounds/time_up_s.ogg")
                    self.gameView:showAllWords()
                    self._timerShowWords = Timer.new ( 1.5 ):start ( 1 )
                end    
            end;
            
            [ E_MODEL.MOVE_MADE ] = function ( self, sender, event, args )
                if args.correct then
                    if self.isMultiplayer then
                        self.channel:send ( "FR_SCORE", { score = self.model._score, word = args.word } )
                    end
                end
            end;
            
            
            [ E_TIMEOUT ] = function ( self, sender, event, args )
                if sender == self._timerShowWords then
                    self.gameGameOverView = GameOverDialog.new ( self.model )
                    self.gameGameOverView:show () 
                end
            end;            

            [ E_NET.FR_FINISH ] = function ( self, sender, event, args )
                self.model:setFriendFinish ( args.score, args.all_words )
                if self.gameGameOverView ~= nil then
                    self.gameGameOverView:updateDialog ()
                end
            end;
            
            [ E_NET.FR_SCORE ] = function ( self, sender, event, args )
                self.model:setFriendScore ( args.score )
            end;
            

            leave = function ( self )
                self:cleanup ()

                if self.timer then
                    self.timer:stop ()
                    self.timer = nil
                end
            end;

            [ gui.E_ACTION ] = function ( self, sender, event, args )
                print ( "action", args.id )
                if args.id == "GameOverDialog.Ok" then
                    self.gameGameOverView:hide ()
                    self.gameGameOverView = nil
                    return "FINAL"
                end
            end;
        };

        FINAL = 
        {
            enter = function ( self )
                if MOAISim.exit then
                    MOAISim.exit ()
                else
                    os.exit ()
                end
            end;
        };
    }
end

function AppCtrl:activateCheatAllWords ()
    self.gameView.timer:addCheatTime ()
    self.gameView:showAllWords ()
end


function AppCtrl:cleanup ()
    for i, player in ipairs ( self.players ) do
        -- unbind from events
        player:dtor ()
    end
    self.players = {}
end
------------------------------------------------------------------------------------
