

E_MODEL = Event.declare "Model"
    :types
    {
        STATE_CHANGED   = 1,
        MOVE_MADE       = 2,
        FRIEND_MOVES    = 3,
    }

Model = core.class ( Object )
{
    STATE_INIT              = 0,
    STATE_TURN_BEG          = 1,
    STATE_TURN              = 2,
    STATE_TURN_COMPLETE     = 3,
    STATE_TURN_END          = 4,
    STATE_DONE              = 5;

    MIN_LENGTH = 3,
    MAX_LENGTH = 6;
}

function Model:ctor ()
    self.state = self.STATE_INIT
    self.moveMade = core.signal ()
    self.playersInfo =
    {
        [ Parties.PLAYER_1 ] = {},
        [ Parties.PLAYER_2 ] = {},
    }

    self.CHEATWord = 0
    self._score = 0
    self._friend_score = 0
    self._friend_finish = false
    self._time_left = 0

    local words = require "src.model.words"
    local dictionary = {}

    dictionary.all = {}
    dictionary.byLength = {}

    for _, word in ipairs ( words ) do
        dictionary.all [ word ] = true

        local length = string.len ( word )
        local list = dictionary.byLength [ length ] or {}
        table.insert ( list, word )
        dictionary.byLength [ length ] = list
    end

    function dictionary:contains ( word )
        return self.all [ word ]
    end

    self.dictionary = dictionary
    
    app.queue:subscribe ( self, self.onRoundTimeOut, E_ROUND_TIMER.TIME_OUT, nil )
    app.queue:subscribe ( self, self.onRoundTimeTic, E_ROUND_TIMER.TIME_TIC, nil )
end

function Model:onRoundTimeOut ( sender, event, args )
    self:_setState(self.STATE_DONE)
end

function Model:onRoundTimeTic ( sender, event, args )
   self._time_left = args.time
end

function Model:setup ( )
    assert ( self.state == self.STATE_INIT )
    --self.position = Position.new ()
    self.currPlayer = Parties.PLAYER_1
end

function Model:getRandWordKey ()
  --  return "squits"
    local words = self.dictionary.byLength [ 6 ]
    local word = words[ math.random ( 1, #words )]
    local letters = {}
    for i = 1, string.len ( word ) do
        table.insert ( letters, string.sub ( word, i, i ))
    end
    for i = 1, #letters do
        local j = math.random ( i )
        letters [ i ], letters [ j ] = letters [ j ], letters [ i ]
    end
    return table.concat ( letters )
end

function Model:start ( seed )
    assert ( self.state == self.STATE_INIT )

    if not seed then
        -- pick a seed word at random
        local longestWords = self.dictionary.byLength [ self.MAX_LENGTH ]
        longestWords = { "cabman", "abacas", "abases", "abduct", "gritty", "grivet", "grizes" }

        seed = longestWords [1]

 --       seed = longestWords [ math.random ( 1, #longestWords )]
    end

    -- find subwords
    local dictionary = self.dictionary
    local words = {}

    local letters = {}
    for i = 1, string.len ( seed ) do
        local ch = string.sub ( seed, i, i )
        table.insert ( letters, ch )
    end

    local function swap ( table, i, j )
        table [ i ], table [ j ] = table [ j ], table [ i ] 
    end

    local function permute ( elements, func, depth )
        depth = depth or #elements
        if depth == 0 then
            func ( elements )
            return
        end

        for i = 1, depth do
            swap ( elements, i, depth )
            permute ( elements, func, depth - 1 )
            swap ( elements, i, depth )
        end
    end

    permute ( letters,
        function ( letters )
            local s = table.concat ( letters, "" )
            local len = string.len ( s )

            -- scan all prefixes
            for i = 1, len do
                local w = string.sub ( s, 1, i )
                if dictionary:contains ( w ) then
                    words [ w ] = true
                end
            end
        end
    )

    -- save found words in a list
    local wordList = {}
    for w, _ in pairs ( words ) do
        table.insert ( wordList, w )
    end

    table.sort ( wordList, 
        function ( w1, w2 )
            if #w1 ~= #w2 then
                return #w1 < #w2
            end
            return w1 < w2
        end
    )  

    self._seed = seed
    self._wordList = wordList
    self._wordDict = words

    -- debug found words
    log ( "Seed word: %s", seed )
    for _, w in ipairs ( wordList ) do
        log ( "%02d | %s ", _, w )
    end

    self:_setState ( self.STATE_TURN_BEG )
end

function Model:_setState ( nextState )
    local prevState = self.state
    self.state = nextState
    -- log ( "Model.state %d -> %d", prevState, nextState )
    app.queue:postEvent ( self, E_MODEL.STATE_CHANGED, { prevState = prevState, nextState = nextState })
end

function Model:startTurn ()
    assert ( self.state == self.STATE_TURN_BEG )
    -- local position = self.position
    -- self.legalMoves = position:findMoves ( self.currPlayer )

    -- if #self.legalMoves == 0 then
    --     self:_setState ( self.STATE_TURN_COMPLETE )
    -- else
    --     self:_setState ( self.STATE_TURN )
    -- end
    self:_setState ( self.STATE_TURN )
end

function Model:finishTurn ()
    assert ( self.state == self.STATE_TURN_COMPLETE )
    self:_setState ( self.STATE_TURN_END)
end

function Model:nextTurn ()
    assert ( self.state == self.STATE_TURN_END )

    -- if self.position:isTerminal () then
    --     self:_setState ( self.STATE_DONE )
    --     return
    -- end

    self.currPlayer = - self.currPlayer
    self:_setState ( self.STATE_TURN_BEG )
end

function Model:makeMove ( move )
    assert ( self.state == self.STATE_TURN )
    
    -- self.position:makeMove ( self.currPlayer, move )
    self.moveMade ( move )
    app.queue:postEvent ( self, E_MODEL.MOVE_MADE, { move = move })
    
    self:_setState ( self.STATE_TURN_COMPLETE )
    -- self.legalMoves = nil
end

function Model:getSeed ()
    return self._seed
end

function Model:getWordList ()
    return self._wordList
end

function Model:getWordDict ()
    return self._wordDict
end

function Model:getWinner ()
    assert ( self.state == self.STATE_DONE )
    -- local score1 = self.position:getDiscCnt ( Parties.PLAYER_1 )
    -- local score2 = self.position:getDiscCnt ( Parties.PLAYER_2 )

    -- if score1 == score2 then return nil end
    -- if score1 > score2 then return Parties.PLAYER_1 end
    -- if score1 < score2 then return Parties.PLAYER_2 end

    return self.currPlayer
end


function Model:submitWord ( word )
    local isWordCorrect = self._wordDict[word];
    
    if isWordCorrect then
        self._wordDict[word] = false
        self._score = self._score + string.len (word)
        if self:isFindAllWords () then
            self._score = self._score + math.floor ( self._time_left * 0.5 )
            self:_setState (self.STATE_DONE)    
        end
    end
    
    app.queue:postEvent ( self, E_MODEL.MOVE_MADE, { correct = isWordCorrect, word = word } )
    
    return isWordCorrect
end

function Model:isFindAllWords ()
    
   -- return true
    local findAllWords = true
    for n,p in pairs( self._wordDict ) do
        if p then
           findAllWords = false
           break
        end
    end
    return findAllWords;
end

function Model:setFriendFinish ( _score, _all_words )
    self._friend_score = _score
    self._friend_finish = true
    if _all_words then
        Timer.new ( 0.1,
            function ()
                self:_setState(self.STATE_DONE)  
            end
        ):start ( 1 )
    end
end

function Model:getFriendFinish ()
    --return true
    return self._friend_finish
end

function Model:getPlayerScore ()
    return self._score
end

function Model:getFriendScore ()
   -- return 150
   return self._friend_score
end

function Model:setFriendScore( _score )
    self._friend_score = _score
    app.queue:postEvent ( self, E_MODEL.FRIEND_MOVES, { })
end

function Model:getCHEATNextWord()
  self.CHEATWord = self.CHEATWord + 1
  
  if self.CHEATWord > #self._wordList then
      self.CHEATWord = #self._wordList
  end
  
  print(self.CHEATWord)
  return self._wordList[self.CHEATWord]
end
        

