GameView = core.class ( gui.Scene )
{
    E_ANIMATED = Event.declare "GameView.animated"
        :types
        {
            START_GAME      = 0,
            MOVE            = 1,
            SPLASH_DONE     = 2,
        };
}

function GameView:ctor ( model )
    local root = rm.RSRC_GAME_VIEW:create ()
    gui.Scene.ctor ( self, root )

    self._inputListeners =
    {
        self,
        root.btnExit, root.btnInfo,
        root.btnSubmit, root.btnShuffle,
    }

    self.model = model

    self.moveSelected = core.signal ()
  
    self.board = WordBoard.new
    {
        x = -245 * WORD_BOARD_SCALE, y = 310 + TOP_PANEL_OFFSET_Y;
        scale = WORD_BOARD_SCALE,
    }:attach ( self.root )

    self.inputBoard = InputBoard.new
    {
        x = -242 * WORD_BOARD_SCALE, y = -310 - TOP_PANEL_OFFSET_Y  - INPUT_BOARD_OFFSET_Y ;
        scale = WORD_BOARD_SCALE,
    }:attach ( self.root )

    self.playerScore = PlayerScore.new
    {
        x = -105, y = 405 + TOP_PANEL_OFFSET_Y;
    }:attach ( self.root )

    self.friendScore = PlayerScore.new
    {
        x = 55, y = 405 + TOP_PANEL_OFFSET_Y;
    }:attach ( self.root )

    self.timer = GameTimer.new
    {
        x = -244, y = -425 - TOP_PANEL_OFFSET_Y;
    }:attach ( self.root )    

    self.wdgCountDown = rm.RSRC_COUNTDOWN_VIEW:create () :attach ( self.root )

    app.queue:subscribe ( self, self.onModelEvent, E_MODEL, model )
    app.queue:subscribe ( self, self.onAction, gui.E_ACTION )
    
    app.queue:subscribe ( self, self.onHideSplash, GameView.E_ANIMATED.SPLASH_DONE )
    

    self.inputBlocked = 0
end

function GameView:getInputListeners ()
    if self.inputBlocked == 0 then
        return self._inputListeners
    end
end

function GameView:onViewportChanged ( sender, event, args )
    -- adjust subviews
end

function GameView:startSplash ( isMultiplayer )
    self.splashScreen = SplashScreen.new (
        {
            x = 0, y = 0;
        },
        nil,
        "words" )
        :attach ( self.root )
    av:playSound("art/sounds/shuffle_s.ogg")
    
    self:hideControl ( self.root.scoreInfoYou )
    self:hideControl ( self.root.scoreInfoFriend )
    
end

function GameView:splashShowWaitMsg ()
    if self.splashScreen ~= nil  then
        self.splashScreen:showWaitMsg ()
    end
end

function GameView:hideControl ( ctrl )
    local xLoc, yLoc  = ctrl.transform:getLoc ();
    ctrl.transform:setLoc(xLoc, yLoc + 1200 );
end

function GameView:showControl ( ctrl )
    local xLoc, yLoc  = ctrl.transform:getLoc ();
    ctrl.transform:setLoc(xLoc, yLoc - 1200 );
end

function GameView:onHideSplash ( sender, event, args  )
    print("ON HIDE")
    if self.splashScreen ~= nil then
        self.splashScreen:detach ()
        self.splashScreen = nil
    end
end

function GameView:hideSplash ( )

    if self.splashScreen ~= nil then
        self.splashScreen:hide ()
    end
     
    self:showControl ( self.root.scoreInfoYou )
    self.root.scoreInfoYou.prop:seekColor ( 0, 0, 0, 0.0, 0.0)
    
    self:showControl ( self.root.scoreInfoFriend )
    self.root.scoreInfoFriend.prop:seekColor ( 0, 0, 0, 0.0, 0.0)
        
    self.playerScore:hide()

    self.friendScore:hide()
    
    self.timer:hide()
end


function GameView:startGame ()
    -- make transition from splash to the

    self.root.scoreInfoYou.prop:seekColor ( 1, 1, 1, 1.0, 1.5 )
    if self.model.isMultiplayer then
        self.root.scoreInfoFriend.prop:seekColor ( 1, 1, 1, 1.0, 1.5 )
    end
    self.playerScore:show ( 1.5 )
    
    if self.model.isMultiplayer then
        self.friendScore:show ( 1.5 )
    end
    
    self.timer:show ( 1.5 )

    local counter = 1
    local wdgCounter = self.wdgCountDown
    local imgCounter = self.wdgCountDown.imgCounter
  --  local txtCounter = self.root.txtCounter
  
  

    local function present ( idx, fadeOut )
--        text = tostring ( text )
 --       wdgCounter.transform:setScl ( 1.0 )
  ---      wdgCounter.transform:seekScl ( 2.0, 2.0, 1.0, MOAIEaseType.SOFT_EASE_OUT )
           imgCounter.prop:setIndex ( idx )
           av:playSound("art/sounds/round_tick3_s.ogg")
           
           if counter == 3 then
                Timer.new ( 1.0,
                    function ()
                        av:playSound("art/sounds/round_tick4_s.ogg")
                      --  av:playSound("art/sounds/round_tick3_s.ogg")
                    end
                ):start ( 1 )
           end

           
    --      print("=set", idx)

 --       txtCounter.prop:setString ( text )
  --      txtCounter.prop:setColor ( 1, 1, 1, 1 )
  --      txtCounter.prop:seekColor ( 1, 1, 1, 0, 1.0 )
    end


    present ( counter )
    Timer.new ( 1.0,
        function ()
            counter = counter + 1
            if counter <= 3 then
                present ( counter )
            else
                --if counter == 0 then
        --        present ( "GO!", true )
        --    else
--                av:playSound("art/sounds/round_tick3_s.ogg")
                
        
                
                self.timer:start ()
                app.queue:postEvent ( self, self.E_ANIMATED.START_GAME, {})
               -- 
               
                local action = wdgCounter.transform:moveLoc ( 0, 1200, 0.75 )
                action:setListener ( MOAIAction.EVENT_STOP, 
                    function ()
                        wdgCounter:detach ()    
                    end
                )
            end
        end
    ):start ( 3 )
end

function GameView:setInteractive ( interactive )
    self.isInteractive = interactive
end

function GameView:onTouchBegan ( sender, event, args )
    local x, y = args.x, args.y

    if self.inputBlocked > 0 then
        return
    end

    if not self.isInteractive then
        return
    end

    -- self.moveSelected ( 42 )
end

function GameView:showAllWords ()
    local words_to_show = {}
    local word_dict = self.model:getWordDict()
    for n,p in pairs( word_dict ) do
        if p then
            table.insert(words_to_show,n)
        end
    end
    self.board:showWords( words_to_show )
end


function GameView:onAction ( sender, event, args )
    if args.id == "ExitDialog.BtnYes" then
        -- exit
    end
    
    if self.splashScreen ~= nil then
        return
    end
    
    if self.model.state == self.model.STATE_DONE or self.model.state == self.model.STATE_INIT then
        return
    end
    
    if string.find(args.id, "GameView.Char1_") then
        local len = string.len ( args.id )
        local n = string.byte(args.id, len ) - string.byte( "1" ) + 1;
        self.inputBoard:addLetter(n)
        av:playSound("art/sounds/press_letter_s.ogg")
      --  av:playSound("art/sounds/shuffle_s.ogg")
    end
    
    if string.find(args.id, "GameView.Char2_") then
        local len = string.len ( args.id )
        local n = string.byte(args.id, len ) - string.byte( "1" ) + 1;
        self.inputBoard:remLetter(n)
        av:playSound("art/sounds/press_letter_s.ogg")
    end
    
    
    if args.id == "GameView.Submit" then
        local word = self.inputBoard:getWord()
        if self.model:submitWord ( word ) then
            self.board:showWord ( word, false )
          --  self.board:markFriendWord ( word )
            self.playerScore:setScore (self.model:getPlayerScore ())
        end 
        self.inputBoard:clearWord ()
      --  av:playSound("art/sounds/game_win_s.ogg")
    end
  

    if args.id == "GameView.Shuffle" then
        --self.inputBoard:detach ()
      --   self.inputBoard:clearWord()
      --   self.sounds:play("shuffle_s.ogg")
         if self.inputBoard:Shuffle() then
            av:playSound("art/sounds/press_letter_s.ogg")
       --     av:playSound("art/sounds/game_lose_s.ogg")
         end
    end
    -- if  args.id == "GameView.Exit" and not self.popup then
    --     self.inputBlocked = self.inputBlocked + 1
    --     self.popup = ExitDialog.new ()
    --     self.popup:open ()
    -- end

    -- if  args.id == "GameView.Info" and not self.popup then
    --     self.inputBlocked = self.inputBlocked + 1
    --     self.popup = HelpDialog.new ()
    --     self.popup:open ()
    -- end    

    -- if args.id == "ExitDialog.BtnNo" or args.id == "HelpDialog.Done" then
    --     self.popup:close ()
    --     self.popup = nil
    --     self.inputBlocked = self.inputBlocked - 1
    -- end
end

function GameView:onModelEvent ( sender, event, args )
    local model = sender
    if event == E_MODEL.MOVE_MADE then
        local function onComplete ()
            app.queue:postEvent ( self, self.E_ANIMATED.MOVE, {} )
        end
        onComplete ()
    elseif event == E_MODEL.FRIEND_MOVES then
        self.friendScore:setScore (self.model:getFriendScore ())
    elseif event == E_MODEL.STATE_CHANGED then
        if args.prevState == model.STATE_INIT then
            local seed = model:getSeed ()
            local wordList = model:getWordList ()

            self.board:createBoard ( wordList )
            self.inputBoard:createInput ( seed )
            for _,p in ipairs ( self.inputBoard:getBtnCtrls() ) do
                table.insert ( self._inputListeners, p )
            end
          
            --self:showAllWords ()
            -- self.players [ Parties.PLAYER_1 ]:updateName ():updateScore ()
            -- self.players [ Parties.PLAYER_2 ]:updateName ():updateScore ()
        end

        if args.nextState == model.STATE_TURN_BEG then
            -- self.players [ Parties.PLAYER_1 ]:update ()
            -- self.players [ Parties.PLAYER_2 ]:update ()
        end
        
        if args.nextState == model.STATE_DONE then
           self.timer:stop ()  
        end
        
    end
end

function GameView:clearInput ()
    self.inputBoard:clearWord ()
end
